from rest_framework import status

from mayan.apps.documents.permissions import (
    permission_document_type_edit, permission_document_type_view
)
from mayan.apps.documents.tests.mixins import DocumentTestMixin
from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..models import DocumentTypeMetadataType, MetadataType
from ..permissions import (
    permission_document_metadata_add, permission_document_metadata_edit,
    permission_document_metadata_remove, permission_document_metadata_view,
    permission_metadata_type_create, permission_metadata_type_delete,
    permission_metadata_type_edit, permission_metadata_type_view
)

from .literals import (
    TEST_METADATA_TYPE_DEFAULT_VALUE, TEST_METADATA_VALUE,
    TEST_METADATA_VALUE_EDITED
)
from .mixins import MetadataTypeAPIViewTestMixin, MetadataTypeTestMixin


class MetadataTypeAPITestCase(
    MetadataTypeAPIViewTestMixin, MetadataTypeTestMixin, BaseAPITestCase
):
    def test_metadata_type_create_no_permission(self):
        response = self._request_test_metadata_type_create_view()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.assertEqual(MetadataType.objects.count(), 0)

    def test_metadata_type_create_with_permission(self):
        self.grant_permission(permission=permission_metadata_type_create)
        response = self._request_test_metadata_type_create_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        metadata_type = MetadataType.objects.first()
        self.assertEqual(response.data['id'], metadata_type.pk)

    def test_metadata_type_delete_no_access(self):
        self._create_test_metadata_type()
        response = self._request_test_metadata_type_delete_view()

        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertEqual(MetadataType.objects.count(), 1)

    def test_metadata_type_delete_with_access(self):
        self._create_test_metadata_type()
        self.grant_access(
            obj=self.test_metadata_type, permission=permission_metadata_type_delete
        )

        response = self._request_test_metadata_type_delete_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(MetadataType.objects.count(), 0)

    def test_metadata_type_detail_view_no_access(self):
        self._create_test_metadata_type()

        response = self._request_test_metadata_type_detail_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_metadata_type_detail_view_with_access(self):
        self._create_test_metadata_type()
        self.grant_access(
            obj=self.test_metadata_type, permission=permission_metadata_type_view
        )

        response = self._request_test_metadata_type_detail_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            response.data['label'], self.test_metadata_type.label
        )

    def test_metadata_type_patch_view_no_access(self):
        self._create_test_metadata_type()
        metadata_type_values = self._model_instance_to_dictionary(
            instance=self.test_metadata_type
        )

        response = self._request_test_metadata_type_edit_view_via_patch()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.test_metadata_type.refresh_from_db()
        self.assertEqual(
            self._model_instance_to_dictionary(
                instance=self.test_metadata_type
            ), metadata_type_values
        )

    def test_metadata_type_patch_view_with_access(self):
        self._create_test_metadata_type()
        metadata_type_values = self._model_instance_to_dictionary(
            instance=self.test_metadata_type
        )
        self.grant_access(
            obj=self.test_metadata_type, permission=permission_metadata_type_edit
        )

        response = self._request_test_metadata_type_edit_view_via_patch()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.test_metadata_type.refresh_from_db()
        self.assertNotEqual(
            self._model_instance_to_dictionary(
                instance=self.test_metadata_type
            ), metadata_type_values
        )

    def test_metadata_type_put_view_no_access(self):
        self._create_test_metadata_type()
        metadata_type_values = self._model_instance_to_dictionary(
            instance=self.test_metadata_type
        )

        response = self._request_test_metadata_type_edit_view_via_put()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.test_metadata_type.refresh_from_db()
        self.assertEqual(
            self._model_instance_to_dictionary(
                instance=self.test_metadata_type
            ), metadata_type_values
        )

    def test_metadata_type_put_view_with_access(self):
        self._create_test_metadata_type()
        metadata_type_values = self._model_instance_to_dictionary(
            instance=self.test_metadata_type
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_metadata_type_edit
        )

        response = self._request_test_metadata_type_edit_view_via_put()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.test_metadata_type.refresh_from_db()
        self.assertNotEqual(
            self._model_instance_to_dictionary(
                instance=self.test_metadata_type
            ), metadata_type_values
        )

    def test_metadata_type_list_view_no_access(self):
        self._create_test_metadata_type()
        response = self._request_test_metadata_type_list_view()

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 0)

    def test_metadata_type_list_view_with_access(self):
        self._create_test_metadata_type()
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_metadata_type_view
        )

        response = self._request_test_metadata_type_list_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['results'][0]['label'],
            self.test_metadata_type.label
        )


class DocumentTypeMetadataTypeAPITestCase(
    DocumentTestMixin, MetadataTypeTestMixin, BaseAPITestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super(DocumentTypeMetadataTypeAPITestCase, self).setUp()
        self._create_test_metadata_type()

    def _create_test_document_type_metadata_type(self):
        self.test_document_type_metadata_type = self.test_document_type.metadata.create(
            metadata_type=self.test_metadata_type, required=False
        )

    def _request_document_type_metadata_type_create_view(self):
        return self.post(
            viewname='rest_api:documenttypemetadatatype-list',
            kwargs={'document_type_pk': self.test_document_type.pk}, data={
                'metadata_type_pk': self.test_metadata_type.pk, 'required': False
            }
        )

    def test_document_type_metadata_type_create_view_no_access(self):
        response = self._request_document_type_metadata_type_create_view()

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.assertEqual(self.test_document_type.metadata.count(), 0)

    def test_document_type_metadata_type_create_view_with_access(self):
        self.grant_access(
            obj=self.test_document_type,
            permission=permission_document_type_edit
        )

        response = self._request_document_type_metadata_type_create_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        document_type_metadata_type = DocumentTypeMetadataType.objects.first()
        self.assertEqual(response.data['id'], document_type_metadata_type.pk)

    def test_document_type_metadata_type_create_dupicate_view(self):
        self._create_test_document_type_metadata_type()
        self.grant_permission(permission=permission_document_type_edit)
        response = self._request_document_type_metadata_type_create_view()

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(list(response.data.keys())[0], 'non_field_errors')

    def _request_document_type_metadata_type_delete_view(self):
        return self.delete(
            viewname='rest_api:documenttypemetadatatype-detail',
            kwargs={
                'document_type_pk': self.test_document_type.pk,
                'metadata_type_pk': self.test_document_type_metadata_type.pk
            }
        )

    def test_document_type_metadata_type_delete_view_no_access(self):
        self._create_test_document_type_metadata_type()

        response = self._request_document_type_metadata_type_delete_view()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        self.assertEqual(self.test_document_type.metadata.count(), 1)

    def test_document_type_metadata_type_delete_view_with_access(self):
        self._create_test_document_type_metadata_type()
        self.grant_access(
            obj=self.test_document_type,
            permission=permission_document_type_edit
        )

        response = self._request_document_type_metadata_type_delete_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(self.test_document_type.metadata.all().count(), 0)

    def _request_document_type_metadata_type_list_view(self):
        return self.get(
            viewname='rest_api:documenttypemetadatatype-list', kwargs={
                'document_type_pk': self.test_document_type.pk
            }
        )

    def test_document_type_metadata_type_list_view_no_access(self):
        self._create_test_document_type_metadata_type()

        response = self._request_document_type_metadata_type_list_view()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_document_type_metadata_type_list_view_with_access(self):
        self._create_test_document_type_metadata_type()
        self.grant_access(
            obj=self.test_document_type,
            permission=permission_document_type_view
        )

        response = self._request_document_type_metadata_type_list_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['results'][0]['id'],
            self.test_document_type_metadata_type.pk
        )

    def _request_document_type_metadata_type_edit_view_via_patch(self):
        return self.patch(
            viewname='rest_api:documenttypemetadatatype-detail',
            kwargs={
                'document_type_pk': self.test_document_type.pk,
                'metadata_type_pk': self.test_document_type_metadata_type.pk
            }, data={
                'required': True
            }
        )

    def test_document_type_metadata_type_patch_view_no_access(self):
        self._create_test_document_type_metadata_type()

        response = self._request_document_type_metadata_type_edit_view_via_patch()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        document_type_metadata_type = DocumentTypeMetadataType.objects.first()
        self.assertFalse(document_type_metadata_type.required, True)

    def test_document_type_metadata_type_patch_view_with_access(self):
        self._create_test_document_type_metadata_type()
        self.grant_access(
            obj=self.test_document_type,
            permission=permission_document_type_edit
        )

        response = self._request_document_type_metadata_type_edit_view_via_patch()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        document_type_metadata_type = DocumentTypeMetadataType.objects.first()
        self.assertEqual(document_type_metadata_type.required, True)

    def _request_document_type_metadata_type_edit_view_via_put(self):
        return self.put(
            viewname='rest_api:documenttypemetadatatype-detail',
            kwargs={
                'document_type_pk': self.test_document_type.pk,
                'metadata_type_pk': self.test_document_type_metadata_type.pk
            }, data={
                'required': True
            }
        )

    def test_document_type_metadata_type_put_view_no_access(self):
        self._create_test_document_type_metadata_type()

        response = self._request_document_type_metadata_type_edit_view_via_put()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        document_type_metadata_type = DocumentTypeMetadataType.objects.first()
        self.assertFalse(document_type_metadata_type.required, True)

    def test_document_type_metadata_type_put_view_with_access(self):
        self._create_test_document_type_metadata_type()
        self.grant_access(
            obj=self.test_document_type, permission=permission_document_type_edit
        )

        response = self._request_document_type_metadata_type_edit_view_via_put()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        document_type_metadata_type = DocumentTypeMetadataType.objects.first()
        self.assertEqual(document_type_metadata_type.required, True)


class DocumentMetadataAPITestMixin(object):
    def _request_document_metadata_create_view(self, extra_data=None):
        data = {
            'metadata_type_pk': self.test_metadata_type.pk,
            'value': TEST_METADATA_VALUE
        }
        data.update(extra_data or {})

        return self.post(
            viewname='rest_api:documentmetadata-list',
            kwargs={'document_pk': self.test_document.pk}, data=data
        )

    def _request_document_metadata_delete_view(self):
        return self.delete(
            viewname='rest_api:documentmetadata-detail',
            kwargs={
                'document_pk': self.test_document.pk,
                'metadata_pk': self.test_document_metadata.pk
            }
        )

    def _request_document_metadata_edit_view_via_patch(self, extra_data=None):
        data = {
            'value': TEST_METADATA_VALUE_EDITED
        }
        data.update(extra_data or {})

        return self.patch(
            viewname='rest_api:documentmetadata-detail',
            kwargs={
                'document_pk': self.test_document.pk,
                'metadata_pk': self.test_document_metadata.pk
            }, data=data
        )

    def _request_document_metadata_edit_view_via_put(self, extra_data=None):
        data = {
            'value': TEST_METADATA_VALUE_EDITED
        }
        data.update(extra_data or {})

        return self.put(
            viewname='rest_api:documentmetadata-detail',
            kwargs={
                'document_pk': self.test_document.pk,
                'metadata_pk': self.test_document_metadata.pk
            }, data=data
        )

    def _request_document_metadata_list_view(self):
        return self.get(
            viewname='rest_api:documentmetadata-list', kwargs={
                'document_pk': self.test_document.pk
            }
        )


class DocumentMetadataAPITestCase(
    DocumentTestMixin, DocumentMetadataAPITestMixin, MetadataTypeTestMixin,
    BaseAPITestCase
):
    def setUp(self):
        super(DocumentMetadataAPITestCase, self).setUp()
        self._create_test_metadata_type()
        self.test_metadata_type.default = TEST_METADATA_TYPE_DEFAULT_VALUE
        self.test_metadata_type.save()
        self.test_document_type.metadata.create(
            metadata_type=self.test_metadata_type, required=False
        )

    def _create_document_metadata(self):
        self.test_document_metadata = self.test_document.metadata.create(
            metadata_type=self.test_metadata_type, value=TEST_METADATA_VALUE
        )

    def test_document_metadata_create_view_no_access(self):
        response = self._request_document_metadata_create_view()

        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertEqual(self.test_document.metadata.count(), 0)

    def test_document_metadata_create_view_with_document_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_metadata_add
        )

        response = self._request_document_metadata_create_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertEqual(self.test_document.metadata.count(), 0)

    def test_document_metadata_create_view_with_metadata_type_access(self):
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_add
        )

        response = self._request_document_metadata_create_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertEqual(self.test_document.metadata.count(), 0)

    def test_document_metadata_create_view_with_full_access(self):
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_add
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_add
        )

        response = self._request_document_metadata_create_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        document_metadata = self.test_document.metadata.first()
        self.assertEqual(response.data['id'], document_metadata.pk)
        self.assertEqual(document_metadata.metadata_type, self.test_metadata_type)
        self.assertEqual(document_metadata.value, TEST_METADATA_VALUE)

    def test_document_metadata_create_default_value_view_with_full_access(self):
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_add
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_add
        )

        response = self._request_document_metadata_create_view(
            extra_data={'value': ''}
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        document_metadata = self.test_document.metadata.first()
        self.assertEqual(response.data['id'], document_metadata.pk)
        self.assertEqual(
            document_metadata.metadata_type, self.test_metadata_type
        )
        self.assertEqual(
            document_metadata.value, TEST_METADATA_TYPE_DEFAULT_VALUE
        )

    def test_document_metadata_create_duplicate_view(self):
        self._create_document_metadata()
        self.grant_permission(permission=permission_document_metadata_add)

        response = self._request_document_metadata_create_view()
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(list(response.data.keys())[0], 'non_field_errors')

    def test_document_metadata_create_invalid_lookup_value_view(self):
        self.test_metadata_type.lookup = 'invalid,lookup,values,on,purpose'
        self.test_metadata_type.save()
        self.grant_permission(permission=permission_document_metadata_add)

        response = self._request_document_metadata_create_view()
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(list(response.data.keys())[0], 'non_field_errors')

    def test_document_metadata_delete_view_no_access(self):
        self._create_document_metadata()

        response = self._request_document_metadata_delete_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertEqual(self.test_document.metadata.all().count(), 1)

    def test_document_metadata_delete_view_with_document_access(self):
        self._create_document_metadata()
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_remove
        )

        response = self._request_document_metadata_delete_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertEqual(self.test_document.metadata.all().count(), 1)

    def test_document_metadata_delete_view_with_metadata_type_access(self):
        self._create_document_metadata()
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_remove
        )

        response = self._request_document_metadata_delete_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertEqual(self.test_document.metadata.all().count(), 1)

    def test_document_metadata_delete_view_with_full_access(self):
        self._create_document_metadata()
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_remove
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_remove
        )

        response = self._request_document_metadata_delete_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
        self.assertEqual(self.test_document.metadata.all().count(), 0)

    def test_document_metadata_list_view_no_access(self):
        self._create_document_metadata()

        response = self._request_document_metadata_list_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_document_metadata_list_view_with_document_access(self):
        self._create_document_metadata()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_view
        )

        response = self._request_document_metadata_list_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 0)

    def test_document_metadata_list_view_with_metadata_type_access(self):
        self._create_document_metadata()

        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_view
        )

        response = self._request_document_metadata_list_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_document_metadata_list_view_with_full_access(self):
        self._create_document_metadata()
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_view
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_view
        )

        response = self._request_document_metadata_list_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['results'][0]['document']['id'], self.test_document.pk
        )
        self.assertEqual(
            response.data['results'][0]['metadata_type']['id'],
            self.test_metadata_type.pk
        )
        self.assertEqual(
            response.data['results'][0]['value'], TEST_METADATA_VALUE
        )
        self.assertEqual(
            response.data['results'][0]['id'], self.test_document_metadata.pk
        )

    def test_document_metadata_patch_view_no_access(self):
        self._create_document_metadata()

        response = self._request_document_metadata_edit_view_via_patch()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.test_document_metadata.refresh_from_db()
        self.assertEqual(self.test_document_metadata.value, TEST_METADATA_VALUE)

    def test_document_metadata_patch_view_document_access(self):
        self._create_document_metadata()
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_edit
        )

        response = self._request_document_metadata_edit_view_via_patch()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.test_document_metadata.refresh_from_db()
        self.assertEqual(self.test_document_metadata.value, TEST_METADATA_VALUE)

    def test_document_metadata_patch_view_metadata_type_access(self):
        self._create_document_metadata()
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_edit
        )

        response = self._request_document_metadata_edit_view_via_patch()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.test_document_metadata.refresh_from_db()
        self.assertEqual(self.test_document_metadata.value, TEST_METADATA_VALUE)

    def test_document_metadata_patch_view_with_full_access(self):
        self._create_document_metadata()
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_edit
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_edit
        )

        response = self._request_document_metadata_edit_view_via_patch()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.test_document_metadata.refresh_from_db()
        self.assertEqual(
            response.data['value'], TEST_METADATA_VALUE_EDITED
        )
        self.assertEqual(
            self.test_document_metadata.value, TEST_METADATA_VALUE_EDITED
        )

    def test_document_metadata_patch_default_value_view_with_full_access(self):
        self._create_document_metadata()
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_edit
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_edit
        )

        response = self._request_document_metadata_edit_view_via_patch(
            extra_data={'value': ''}
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.test_document_metadata.refresh_from_db()
        self.assertEqual(
            response.data['value'], TEST_METADATA_TYPE_DEFAULT_VALUE
        )
        self.assertEqual(
            self.test_document_metadata.value, TEST_METADATA_TYPE_DEFAULT_VALUE
        )

    def test_document_metadata_put_view_no_access(self):
        self._create_document_metadata()

        response = self._request_document_metadata_edit_view_via_put()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.test_document_metadata.refresh_from_db()
        self.assertEqual(self.test_document_metadata.value, TEST_METADATA_VALUE)

    def test_document_metadata_put_view_document_access(self):
        self._create_document_metadata()
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_edit
        )

        response = self._request_document_metadata_edit_view_via_put()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.test_document_metadata.refresh_from_db()
        self.assertEqual(self.test_document_metadata.value, TEST_METADATA_VALUE)

    def test_document_metadata_put_view_metadata_type_access(self):
        self._create_document_metadata()
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_edit
        )

        response = self._request_document_metadata_edit_view_via_put()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.test_document_metadata.refresh_from_db()
        self.assertEqual(self.test_document_metadata.value, TEST_METADATA_VALUE)

    def test_document_metadata_put_view_with_full_access(self):
        self._create_document_metadata()
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_edit
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_edit
        )

        response = self._request_document_metadata_edit_view_via_put()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.test_document_metadata.refresh_from_db()
        self.assertEqual(
            response.data['value'], TEST_METADATA_VALUE_EDITED
        )
        self.assertEqual(
            self.test_document_metadata.value, TEST_METADATA_VALUE_EDITED
        )

    def test_document_metadata_put_default_value_view_with_full_access(self):
        self._create_document_metadata()
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_edit
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_edit
        )

        response = self._request_document_metadata_edit_view_via_put(
            extra_data={'value': ''}
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.test_document_metadata.refresh_from_db()
        self.assertEqual(
            response.data['value'], TEST_METADATA_TYPE_DEFAULT_VALUE
        )
        self.assertEqual(
            self.test_document_metadata.value, TEST_METADATA_TYPE_DEFAULT_VALUE
        )



import logging
import uuid

from django.apps import apps
from django.core.files import File
from django.db import models, transaction
from django.urls import reverse
from django.utils.encoding import python_2_unicode_compatible
from django.utils.timezone import now
from django.utils.translation import ugettext, ugettext_lazy as _

from mayan.apps.common.signals import signal_mayan_pre_save

from ..events import (
    event_document_create, event_document_properties_edit,
    event_document_trashed, event_document_type_changed,
)
from ..literals import DEFAULT_LANGUAGE
from ..managers import DocumentManager, PassthroughManager, TrashCanManager
from ..signals import post_document_type_change

from .document_type_models import DocumentType

__all__ = ('Document',)
logger = logging.getLogger(name=__name__)


class HooksMixin(object):
    @classmethod
    def _execute_hooks(cls, hook_list, **kwargs):
        result = None

        for hook in hook_list:
            result = hook(**kwargs)
            if result:
                kwargs.update(result)

        return result

    @classmethod
    def _insert_hook_entry(cls, hook_list, func, order=None):
        order = order or len(hook_list)
        hook_list.insert(order, func)


@python_2_unicode_compatible
class Document(HooksMixin, models.Model):
    """
    Defines a single document with it's fields and properties
    Fields:
    * uuid - UUID of a document, universally Unique ID. An unique identifier
    generated for each document. No two documents can ever have the same UUID.
    This ID is generated automatically.
    """
    _hooks_pre_create = []

    uuid = models.UUIDField(
        default=uuid.uuid4, editable=False, help_text=_(
            'UUID of a document, universally Unique ID. An unique identifier '
            'generated for each document.'
        ), verbose_name=_('UUID')
    )
    document_type = models.ForeignKey(
        help_text=_('The document type of the document.'),
        on_delete=models.CASCADE, related_name='documents', to=DocumentType,
        verbose_name=_('Document type')
    )
    label = models.CharField(
        blank=True, db_index=True, default='', max_length=255,
        help_text=_('The name of the document.'), verbose_name=_('Label')
    )
    description = models.TextField(
        blank=True, default='', help_text=_(
            'An optional short text describing a document.'
        ), verbose_name=_('Description')
    )
    date_added = models.DateTimeField(
        auto_now_add=True, db_index=True, help_text=_(
            'The server date and time when the document was finally '
            'processed and added to the system.'
        ), verbose_name=_('Added')
    )
    language = models.CharField(
        blank=True, default=DEFAULT_LANGUAGE, help_text=_(
            'The dominant language in the document.'
        ), max_length=8, verbose_name=_('Language')
    )
    in_trash = models.BooleanField(
        db_index=True, default=False, help_text=_(
            'Whether or not this document is in the trash.'
        ), editable=False, verbose_name=_('In trash?')
    )
    deleted_date_time = models.DateTimeField(
        blank=True, editable=True, help_text=_(
            'The server date and time when the document was moved to the '
            'trash.'
        ), null=True, verbose_name=_('Date and time trashed')
    )
    is_stub = models.BooleanField(
        db_index=True, default=True, editable=False, help_text=_(
            'A document stub is a document with an entry on the database but '
            'no file uploaded. This could be an interrupted upload or a '
            'deferred upload via the API.'
        ), verbose_name=_('Is stub?')
    )

    objects = DocumentManager()
    passthrough = PassthroughManager()
    trash = TrashCanManager()

    @classmethod
    def execute_pre_create_hooks(cls, kwargs=None):
        """
        Helper method to allow checking if it is possible to create
        a new document.
        """
        cls._execute_hooks(
            hook_list=cls._hooks_pre_create, kwargs=kwargs
        )

    @classmethod
    def register_pre_create_hook(cls, func, order=None):
        cls._insert_hook_entry(
            hook_list=cls._hooks_pre_create, func=func, order=order
        )

    class Meta:
        ordering = ('label',)
        verbose_name = _('Document')
        verbose_name_plural = _('Documents')

    def __str__(self):
        return self.label or ugettext('Document stub, id: %d') % self.pk

    def add_as_recent_document_for_user(self, user):
        RecentDocument = apps.get_model(
            app_label='documents', model_name='RecentDocument'
        )
        return RecentDocument.objects.add_document_for_user(user, self)

    @property
    def checksum(self):
        return self.latest_version.checksum

    @property
    def date_updated(self):
        return self.latest_version.timestamp

    def delete(self, *args, **kwargs):
        to_trash = kwargs.pop('to_trash', True)
        _user = kwargs.pop('_user', None)

        if not self.in_trash and to_trash:
            self.in_trash = True
            self.deleted_date_time = now()
            with transaction.atomic():
                self.save(_commit_events=False)
                event_document_trashed.commit(actor=_user, target=self)
        else:
            with transaction.atomic():
                for version in self.versions.all():
                    version.delete()

                return super(Document, self).delete(*args, **kwargs)

    def exists(self):
        """
        Returns a boolean value that indicates if the document's
        latest version file exists in storage
        """
        latest_version = self.latest_version
        if latest_version:
            return latest_version.exists()
        else:
            return False

    @property
    def file_mime_encoding(self):
        return self.latest_version.encoding

    @property
    def file_mimetype(self):
        return self.latest_version.mimetype

    def get_absolute_url(self):
        return reverse(
            viewname='documents:document_preview', kwargs={
                'document_id': self.pk
            }
        )

    def get_api_image_url(self, *args, **kwargs):
        latest_version = self.latest_version
        if latest_version:
            return latest_version.get_api_image_url(*args, **kwargs)

    @property
    def is_in_trash(self):
        return self.in_trash

    @property
    def latest_version(self):
        return self.versions.order_by('timestamp').last()

    def natural_key(self):
        return (self.uuid,)
    natural_key.dependencies = ['documents.DocumentType']

    def new_version(self, file_object, comment=None, _user=None):
        logger.info('Creating new document version for document: %s', self)
        DocumentVersion = apps.get_model(
            app_label='documents', model_name='DocumentVersion'
        )

        document_version = DocumentVersion(
            document=self, comment=comment or '', file=File(file_object)
        )
        document_version.save(_user=_user)

        logger.info('New document version queued for document: %s', self)
        return document_version

    def open(self, *args, **kwargs):
        """
        Return a file descriptor to a document's file irrespective of
        the storage backend
        """
        return self.latest_version.open(*args, **kwargs)

    @property
    def page_count(self):
        return self.latest_version.page_count

    @property
    def pages(self):
        try:
            return self.latest_version.pages
        except AttributeError:
            # Document has no version yet
            DocumentPage = apps.get_model(
                app_label='documents', model_name='DocumentPage'
            )

            return DocumentPage.objects.none()

    @property
    def pages_all(self):
        try:
            return self.latest_version.pages_all
        except AttributeError:
            # Document has no version yet
            DocumentPage = apps.get_model(
                app_label='documents', model_name='DocumentPage'
            )

            return DocumentPage.objects.none()

    def restore(self):
        self.in_trash = False
        self.save()

    def save(self, *args, **kwargs):
        user = kwargs.pop('_user', None)
        _commit_events = kwargs.pop('_commit_events', True)
        new_document = not self.pk
        with transaction.atomic():
            signal_mayan_pre_save.send(
                sender=Document, instance=self, user=user
            )

            super(Document, self).save(*args, **kwargs)

            if new_document:
                if user:
                    self.add_as_recent_document_for_user(user=user)
                    event_document_create.commit(
                        actor=user, target=self, action_object=self.document_type
                    )
                else:
                    event_document_create.commit(
                        target=self, action_object=self.document_type
                    )
            else:
                if _commit_events:
                    event_document_properties_edit.commit(actor=user, target=self)

    def save_to_file(self, *args, **kwargs):
        return self.latest_version.save_to_file(*args, **kwargs)

    def set_document_type(self, document_type, force=False, _user=None):
        has_changed = self.document_type != document_type

        self.document_type = document_type
        with transaction.atomic():
            self.save()
            if has_changed or force:
                post_document_type_change.send(
                    sender=self.__class__, instance=self
                )

                event_document_type_changed.commit(actor=_user, target=self)
                if _user:
                    self.add_as_recent_document_for_user(user=_user)

    @property
    def size(self):
        return self.latest_version.size


class TrashedDocument(Document):
    objects = TrashCanManager()

    class Meta:
        proxy = True



import logging

from django.apps import apps

from mayan.celery import app

logger = logging.getLogger(name=__name__)


@app.task()
def task_generate_workflow_image(document_state_id):
    Workflow = apps.get_model(
        app_label='document_states', model_name='Workflow'
    )

    workflow = Workflow.objects.get(pk=document_state_id)

    return workflow.generate_image()


@app.task(ignore_result=True)
def task_launch_all_workflows():
    Document = apps.get_model(app_label='documents', model_name='Document')
    Workflow = apps.get_model(
        app_label='document_states', model_name='Workflow'
    )

    logger.info('Start launching workflows')
    for document in Document.objects.all():
        Workflow.objects.launch_for(document=document)

    logger.info('Finished launching workflows')


@app.task(ignore_result=True)
def task_launch_workflow(workflow_id):
    Document = apps.get_model(app_label='documents', model_name='Document')
    Workflow = apps.get_model(
        app_label='document_states', model_name='Workflow'
    )

    workflow = Workflow.objects.get(pk=workflow_id)

    logger.info('Start launching workflow: %d', workflow_id)
    for document in Document.objects.filter(document_type__in=workflow.document_types.all()):
        workflow.launch_for(document=document)

    logger.info('Finished launching workflow: %d', workflow_id)



import logging

from django.apps import apps
from django.contrib.auth import get_user_model

from mayan.celery import app

logger = logging.getLogger(name=__name__)


@app.task(ignore_result=True)
def task_cache_purge(cache_id, user_id=None):
    Cache = apps.get_model(
        app_label='file_caching', model_name='Cache'
    )
    User = get_user_model()

    cache = Cache.objects.get(pk=cache_id)
    user = User.objects.get(pk=user_id)

    logger.info('Starting cache id %s purge', cache)
    cache.purge(_user=user)
    logger.info('Finished cache id %s purge', cache)



from django.utils.translation import ugettext_lazy as _


DEFAULT_DOCUMENT_BODY_TEMPLATE = _(
    'Attached to this email is the document: {{ document }}\n\n '
    '--------\n '
    'This email has been sent from %(project_title)s (%(project_website)s)'
)
DEFAULT_DOCUMENT_SUBJECT_TEMPLATE = _('Document: {{ document }}')
DEFAULT_LINK_BODY_TEMPLATE = _(
    'To access this document click on the following link: '
    '{{ link }}\n\n--------\n '
    'This email has been sent from %(project_title)s (%(project_website)s)'
)
DEFAULT_LINK_SUBJECT_TEMPLATE = _('Link for document: {{ document }}')
EMAIL_SEPARATORS = (',', ';')



from importlib import import_module
import logging

from django.apps import apps
from django.utils.encoding import force_text

logger = logging.getLogger(name=__name__)


class ModuleLoaderMixin(object):
    _loader_module_name = None

    @classmethod
    def initialize(cls):
        for app in apps.get_app_configs():
            try:
                import_module('{}.{}'.format(app.name, cls._loader_module_name))
            except ImportError as exception:
                if force_text(exception) not in ('No module named {module_name}', 'No module named \'{app_label}.{module_name}\''.format(app_label=app.name, module_name=cls._loader_module_name)):
                    logger.error(
                        'Error importing %s %s.py file; %s',
                        app.name, cls._loader_module_name, exception
                    )
                    raise



from django.contrib.contenttypes.models import ContentType

from rest_framework.permissions import IsAuthenticated

from mayan.apps.rest_api import generics

from .classes import Template
from .serializers import ContentTypeSerializer, TemplateSerializer


class APIContentTypeList(generics.ListAPIView):
    """
    Returns a list of all the available content types.
    """
    serializer_class = ContentTypeSerializer
    queryset = ContentType.objects.order_by('app_label', 'model')


class APITemplateDetailView(generics.RetrieveAPIView):
    """
    Returns the selected partial template details.
    get: Retrieve the details of the partial template.
    """
    serializer_class = TemplateSerializer
    permission_classes = (IsAuthenticated,)

    def get_object(self):
        return Template.get(self.kwargs['name']).render(request=self.request)


class APITemplateListView(generics.ListAPIView):
    """
    Returns a list of all the available templates.
    """
    serializer_class = TemplateSerializer
    permission_classes = (IsAuthenticated,)

    def get_queryset(self):
        return Template.all(rendered=True, request=self.request)



from django.test import override_settings

from rest_framework import status

from mayan.apps.documents.tests.literals import TEST_HYBRID_DOCUMENT
from mayan.apps.documents.tests.mixins import DocumentTestMixin
from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..permissions import (
    permission_content_view, permission_document_type_parsing_setup
)

from .literals import TEST_DOCUMENT_CONTENT
from .mixins import DocumentTypeParsingSettingsAPIViewTestMixin


@override_settings(DOCUMENT_PARSING_AUTO_PARSING=True)
class DocumentParsingAPITestCase(DocumentTestMixin, BaseAPITestCase):
    test_document_filename = TEST_HYBRID_DOCUMENT

    def _request_document_page_content_view(self):
        return self.get(
            viewname='rest_api:document-page-content-view', kwargs={
                'document_pk': self.test_document.pk,
                'version_pk': self.test_document.latest_version.pk,
                'page_pk': self.test_document.latest_version.pages.first().pk
            }
        )

    def test_get_document_version_page_content_no_access(self):
        response = self._request_document_page_content_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_get_document_version_page_content_with_access(self):
        self.grant_access(
            permission=permission_content_view, obj=self.test_document
        )

        response = self._request_document_page_content_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertTrue(
            TEST_DOCUMENT_CONTENT in response.data['content']
        )


class DocumentTypeParsingSettingsAPIViewTestCase(
    DocumentTestMixin, DocumentTypeParsingSettingsAPIViewTestMixin,
    BaseAPITestCase
):
    auto_upload_test_document = False

    def test_document_type_parsing_settings_details_api_view_no_permission(self):
        response = self._request_document_type_parsing_settings_details_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_document_type_parsing_settings_details_api_view_with_access(self):
        self.grant_access(
            obj=self.test_document_type,
            permission=permission_document_type_parsing_setup
        )

        response = self._request_document_type_parsing_settings_details_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, {'auto_parsing': False})

    def test_document_type_parsing_settings_patch_api_view_no_permission(self):
        response = self._request_document_type_parsing_settings_patch_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_document_type_parsing_settings_patch_api_view_with_access(self):
        self.grant_access(
            obj=self.test_document_type,
            permission=permission_document_type_parsing_setup
        )

        response = self._request_document_type_parsing_settings_patch_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, {'auto_parsing': True})

    def test_document_type_parsing_settings_put_api_view_no_permission(self):
        response = self._request_document_type_parsing_settings_put_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_document_type_parsing_settings_put_api_view_with_access(self):
        self.grant_access(
            obj=self.test_document_type,
            permission=permission_document_type_parsing_setup
        )

        response = self._request_document_type_parsing_settings_put_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, {'auto_parsing': True})



import types

from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType
from django.db.models import IntegerField
from django.db.models.functions import Cast
from django.template.defaultfilters import filesizeformat
from django.utils.translation import ugettext_lazy as _

from actstream.models import Action

from mayan.apps.common.signals import signal_mayan_pre_save
from mayan.apps.documents.events import event_document_create
from mayan.apps.documents.models import Document, DocumentVersion
from mayan.apps.user_management.querysets import get_user_queryset

from .classes import QuotaBackend
from .exceptions import QuotaExceeded
from .mixins import DocumentTypesQuotaMixin, GroupsUsersQuotaMixin


def hook_factory_document_check_quota(klass):
    def hook_check_quota(**kwargs):
        # Fake Document to be able to reuse the .process() method
        # for pre check.
        fake_document_instance = types.SimpleNamespace(pk=None)

        final_kwargs = kwargs['kwargs'].copy()
        final_kwargs['instance'] = fake_document_instance

        for quota in klass.get_instances().filter(enabled=True):
            backend_instance = quota.get_backend_instance()

            backend_instance.process(**final_kwargs)
    return hook_check_quota


def hook_factory_document_version_check_quota(klass):
    def hook_check_quota(**kwargs):
        # Pass the real parent document or create a fake one
        if 'document' in kwargs['kwargs']:
            document = kwargs['kwargs']['document']
        else:
            document = types.SimpleNamespace(
                document_type=kwargs['kwargs']['document_type']
            )
        # Fake DocumentVersion to be able to reuse the
        # .process() method for pre check.
        fake_document_instance = types.SimpleNamespace(
            file=kwargs['kwargs']['shared_uploaded_file'].file,
            document=document,
            pk=None
        )

        final_kwargs = kwargs['kwargs'].copy()
        final_kwargs['instance'] = fake_document_instance

        for quota in klass.get_instances().filter(enabled=True):
            backend_instance = quota.get_backend_instance()

            backend_instance.process(**final_kwargs)
    return hook_check_quota


class DocumentCountQuota(
    GroupsUsersQuotaMixin, DocumentTypesQuotaMixin, QuotaBackend
):
    error_message = _('Document count quota exceeded.')
    field_order = ('documents_limit',)
    fields = {
        'documents_limit': {
            'label': _('Documents limit'),
            'class': 'django.forms.IntegerField',
            'help_text': _(
                'Maximum number of documents.'
            )
        },
    }
    label = _('Document count limit')
    sender = Document
    signal = signal_mayan_pre_save

    @classmethod
    def _initialize(cls):
        Document.register_pre_create_hook(
            func=hook_factory_document_check_quota(klass=cls)
        )

    def __init__(
        self, document_type_all, document_type_ids, documents_limit,
        group_ids, user_all, user_ids
    ):
        self.document_type_all = document_type_all
        self.document_type_ids = document_type_ids
        self.documents_limit = documents_limit
        self.group_ids = group_ids
        self.user_all = user_all
        self.user_ids = user_ids

    def _allowed(self):
        return self.documents_limit

    def _allowed_filter_display(self):
        return _('document count: %(document_count)s') % {
            'document_count': self._allowed()
        }

    def _get_user_document_count(self, user):
        action_queryset = Action.objects.annotate(
            target_object_id_int=Cast(
                'target_object_id', output_field=IntegerField()
            ),
        )
        action_filter_kwargs = {
            'verb': event_document_create.id
        }
        document_filter_kwargs = {}

        if not self.document_type_all:
            document_filter_kwargs.update(
                {
                    'document_type_id__in': self._get_document_types().values(
                        'pk'
                    ),
                }
            )

        if user:
            # Admins are always excluded
            if user.is_superuser or user.is_staff:
                return 0

            if not self.user_all:
                users = self._get_users() | get_user_queryset().filter(
                    groups__in=self._get_groups()
                )

                if not users.filter(pk=user.pk).exists():
                    # User is not in the restricted list of users and groups
                    return 0
                else:
                    content_type = ContentType.objects.get_for_model(
                        model=get_user_model()
                    )

                    action_filter_kwargs.update(
                        {
                            'actor_object_id': user.pk,
                            'actor_content_type': content_type,
                        }
                    )

        action_queryset = action_queryset.filter(**action_filter_kwargs)

        document_filter_kwargs.update(
            {
                'pk__in': action_queryset.values('target_object_id_int')
            }
        )

        return Document.objects.filter(**document_filter_kwargs).count()

    def process(self, **kwargs):
        # Only for new documents
        if not kwargs['instance'].pk:
            if self._get_user_document_count(user=kwargs.get('user')) >= self._allowed():
                raise QuotaExceeded(
                    _('Document count quota exceeded.')
                )


class DocumentSizeQuota(
    GroupsUsersQuotaMixin, DocumentTypesQuotaMixin, QuotaBackend
):
    field_order = ('document_size_limit',)
    fields = {
        'document_size_limit': {
            'label': _('Document size limit'),
            'class': 'django.forms.FloatField',
            'help_text': _('Maximum document size in megabytes (MB).')
        }
    }
    label = _('Document size limit')
    sender = DocumentVersion
    signal = signal_mayan_pre_save

    @classmethod
    def _initialize(cls):
        DocumentVersion.register_pre_create_hook(
            func=hook_factory_document_version_check_quota(klass=cls)
        )

    def __init__(
        self, document_size_limit, document_type_all, document_type_ids,
        group_ids, user_all, user_ids
    ):
        self.document_size_limit = document_size_limit
        self.document_type_all = document_type_all
        self.document_type_ids = document_type_ids
        self.group_ids = group_ids
        self.user_all = user_all
        self.user_ids = user_ids

    def _allowed(self):
        return self.document_size_limit * 1024 * 1024

    def _allowed_filter_display(self):
        return _('document size: %(formatted_file_size)s') % {
            'formatted_file_size': filesizeformat(self._allowed())
        }

    def process(self, **kwargs):
        if not kwargs['instance'].pk:
            if kwargs['instance'].file.size >= self._allowed():
                if self.document_type_all or self._get_document_types().filter(pk=kwargs['instance'].document.document_type.pk).exists():
                    # Don't asume there is always a user in the signal.
                    # Non interactive uploads might not include a user.
                    if kwargs['user']:
                        if kwargs['user'].is_superuser or kwargs['user'].is_staff:
                            return

                    users = self._get_users() | get_user_queryset().filter(
                        groups__in=self._get_groups()
                    )

                    if self.user_all or kwargs['user'] and users.filter(pk=kwargs['user'].pk).exists():
                        raise QuotaExceeded(
                            _('Document size quota exceeded.')
                        )



from django.conf.urls import url

from .api_views import APICommentListView, APICommentView
from .views import (
    DocumentCommentCreateView, DocumentCommentDeleteView,
    DocumentCommentDetailView, DocumentCommentEditView,
    DocumentCommentListView
)

urlpatterns = [
    url(
        regex=r'^documents/(?P<document_id>\d+)/comments/$',
        name='comments_for_document', view=DocumentCommentListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/comments/add/$',
        name='comment_add', view=DocumentCommentCreateView.as_view()
    ),
    url(
        regex=r'^comments/(?P<comment_id>\d+)/delete/$',
        name='comment_delete', view=DocumentCommentDeleteView.as_view()
    ),
    url(
        regex=r'^comments/(?P<comment_id>\d+)/$', name='comment_details',
        view=DocumentCommentDetailView.as_view()
    ),
    url(
        regex=r'^comments/(?P<comment_id>\d+)/edit/$', name='comment_edit',
        view=DocumentCommentEditView.as_view()
    ),
]

api_urls = [
    url(
        regex=r'^documents/(?P<document_pk>[0-9]+)/comments/$',
        name='comment-list', view=APICommentListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_pk>[0-9]+)/comments/(?P<comment_pk>[0-9]+)/$',
        name='comment-detail', view=APICommentView.as_view()
    ),
]



from django.utils.encoding import force_text

from rest_framework import status

from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.documents.tests.mixins import DocumentTestMixin
from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..permissions import (
    permission_document_check_in, permission_document_check_in_override,
    permission_document_check_out, permission_document_check_out_detail_view
)

from .mixins import (
    DocumentCheckoutsAPIViewTestMixin, DocumentCheckoutTestMixin
)


class CheckoutsAPITestCase(
    DocumentCheckoutsAPIViewTestMixin, DocumentCheckoutTestMixin,
    DocumentTestMixin, BaseAPITestCase
):
    def test_document_check_out_create_api_view_no_permission(self):
        response = self._request_test_document_check_out_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        self.assertFalse(self.test_document.is_checked_out())

    def test_document_check_out_create_api_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_check_out
        )

        response = self._request_test_document_check_out_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        self.assertTrue(self.test_document.is_checked_out())

    def test_document_check_out_delete_api_view_no_permission(self):
        self._check_out_test_document()

        response = self._request_test_document_check_out_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        self.assertTrue(self.test_document.is_checked_out())

    def test_document_check_out_delete_api_view_with_access(self):
        self._check_out_test_document()

        self.grant_access(
            obj=self.test_document, permission=permission_document_check_in
        )

        response = self._request_test_document_check_out_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertFalse(self.test_document.is_checked_out())

    def test_document_check_in_forcefull_api_view_no_permission(self):
        self._create_test_user()
        self._check_out_test_document(user=self.test_user)

        self.grant_access(
            obj=self.test_document, permission=permission_document_check_in
        )

        response = self._request_test_document_check_out_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        self.assertTrue(self.test_document.is_checked_out())

    def test_document_check_in_forcefull_api_view_with_access(self):
        self._create_test_user()
        self._check_out_test_document(user=self.test_user)

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_check_in_override
        )

        response = self._request_test_document_check_out_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertFalse(self.test_document.is_checked_out())

    def test_document_check_out_detail_api_view_no_permission(self):
        self._check_out_test_document()

        response = self._request_test_document_check_out_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_document_check_out_detail_api_view_with_check_out_detail_access(self):
        self._check_out_test_document()
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_check_out_detail_view
        )

        response = self._request_test_document_check_out_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_document_check_out_detail_api_view_with_document_access(self):
        self._check_out_test_document()
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_document_check_out_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_document_check_out_detail_api_view_with_full_access(self):
        self._check_out_test_document()
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_check_out_detail_view
        )

        response = self._request_test_document_check_out_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['document']['uuid'],
            force_text(self.test_document.uuid)
        )

    def test_document_check_out_list_api_view_no_permission(self):
        self._check_out_test_document()

        response = self._request_test_document_check_out_list_api_view()
        self.assertNotContains(
            response=response, text=self.test_document.uuid,
            status_code=status.HTTP_200_OK
        )

    def test_document_check_out_list_api_view_with_document_access(self):
        self._check_out_test_document()
        self.grant_access(
            permission=permission_document_view, obj=self.test_document
        )

        response = self._request_test_document_check_out_list_api_view()
        self.assertNotContains(
            response=response, text=self.test_document.uuid,
            status_code=status.HTTP_200_OK
        )

    def test_document_check_out_list_api_view_with_check_out_detail_access(self):
        self._check_out_test_document()
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_check_out_detail_view
        )

        response = self._request_test_document_check_out_list_api_view()
        self.assertNotContains(
            response=response, text=self.test_document.uuid,
            status_code=status.HTTP_200_OK
        )

    def test_document_check_out_list_api_view_with_full_access(self):
        self._check_out_test_document()

        self.grant_access(
            permission=permission_document_view, obj=self.test_document
        )
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_check_out_detail_view
        )

        response = self._request_test_document_check_out_list_api_view()
        self.assertContains(
            response=response, text=self.test_document.uuid,
            status_code=status.HTTP_200_OK
        )



# -*- coding: utf-8 -*-

from mayan.apps.dependencies.classes import PythonDependency

PythonDependency(
    copyright_text='''
        dateutil - Extensions to the standard Python datetime module.

        Copyright (c) 2003-2011 - Gustavo Niemeyer <gustavo@niemeyer.net>
        Copyright (c) 2012-2014 - Tomi Pieviläinen <tomi.pievilainen@iki.fi>
        Copyright (c) 2014      - Yaron de Leeuw <me@jarondl.net>

        All rights reserved.

        Redistribution and use in source and binary forms, with or without
        modification, are permitted provided that the following conditions are met:

        * Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright notice,
        this list of conditions and the following disclaimer in the documentation
        and/or other materials provided with the distribution.
        * Neither the name of the copyright holder nor the names of its
        contributors may be used to endorse or promote products derived from
        this software without specific prior written permission.

        THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
        "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
        LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
        A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
        CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
        EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
        PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
        PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
        LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
        NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
        SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    ''', module=__name__, name='python-dateutil', version_string='==2.8.1'
)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.storage.classes import DefinedStorage

from .literals import (
    STORAGE_NAME_DOCUMENT_IMAGE, STORAGE_NAME_DOCUMENT_VERSION
)
from .settings import (
    setting_documentimagecache_storage,
    setting_documentimagecache_storage_arguments,
    setting_storage_backend, setting_storage_backend_arguments
)

storage_document_image_cache = DefinedStorage(
    dotted_path=setting_documentimagecache_storage.value,
    error_message=_(
        'Unable to initialize the document image storage. Check '
        'the settings {} and {} for formatting errors.'.format(
            setting_documentimagecache_storage.global_name,
            setting_documentimagecache_storage_arguments.global_name
        )
    ),
    label=_('Document images'),
    name=STORAGE_NAME_DOCUMENT_IMAGE,
    kwargs=setting_documentimagecache_storage_arguments.value
)

storage_document_versions = DefinedStorage(
    dotted_path=setting_storage_backend.value,
    error_message=_(
        'Unable to initialize the document version storage. Check '
        'the settings {} and {} for formatting errors.'.format(
            setting_storage_backend.global_name,
            setting_storage_backend_arguments.global_name
        )
    ),
    label=_('Document version files'),
    name=STORAGE_NAME_DOCUMENT_VERSION,
    kwargs=setting_storage_backend_arguments.value
)



# Generated by Django 2.2.10 on 2020-03-11 05:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document_states', '0018_auto_20200129_0926'),
    ]

    operations = [
        migrations.AddField(
            model_name='workflowtransition',
            name='condition',
            field=models.TextField(blank=True, help_text="The condition that will determine if this transition is enabled or not. The condition is evaluated against the workflow instance. Condition that return None or an empty string ('') are considered to be logical false, any other value is considered to be the logical true.", verbose_name='Condition'),
        ),
    ]



from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.documents.tests.base import (
    GenericDocumentViewTestCase, GenericViewTestCase
)

from ..models import Index, IndexInstanceNode
from ..permissions import (
    permission_document_indexing_create, permission_document_indexing_delete,
    permission_document_indexing_edit,
    permission_document_indexing_instance_view,
    permission_document_indexing_rebuild
)

from .literals import TEST_INDEX_LABEL, TEST_INDEX_LABEL_EDITED
from .mixins import (
    DocumentIndexViewTestMixin, IndexInstaceViewTestMixin,
    IndexTemplateNodeViewTestMixin, IndexTestMixin, IndexToolsViewTestMixin,
    IndexViewTestMixin
)


class DocumentIndexViewTestCase(
    DocumentIndexViewTestMixin, IndexTestMixin, GenericDocumentViewTestCase
):
    def setUp(self):
        super().setUp()
        self._create_test_index()
        self._create_test_index_template_node()
        self.test_index.document_types.add(self.test_document_type)
        self._upload_test_document()

    def test_document_index_list_view_no_permission(self):
        response = self._request_test_document_index_list_view()
        self.assertEqual(response.status_code, 404)

    def test_document_index_list_view_with_index_access(self):
        self.grant_access(
            obj=self.test_index,
            permission=permission_document_indexing_instance_view
        )

        response = self._request_test_document_index_list_view()
        self.assertEqual(response.status_code, 404)

    def test_document_index_list_view_with_document_access(self):
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_view
        )

        response = self._request_test_document_index_list_view()
        self.assertContains(
            count=0, response=response, status_code=200, text=TEST_INDEX_LABEL
        )
        # 4 instances: title heading, title heading hover, JavaScript
        # document title
        self.assertContains(
            count=3, response=response, status_code=200, text=self.test_document
        )

    def test_document_index_list_view_with_full_access(self):
        self.grant_access(
            obj=self.test_index,
            permission=permission_document_indexing_instance_view
        )
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_view
        )

        response = self._request_test_document_index_list_view()
        self.assertContains(
            count=1, response=response, status_code=200, text=TEST_INDEX_LABEL
        )
        # 4 instances: title heading, title heading hover, JavaScript
        # document title, index entry
        self.assertContains(
            count=4, response=response, status_code=200, text=self.test_document
        )


class IndexViewTestCase(
    IndexTestMixin, IndexViewTestMixin, GenericViewTestCase
):
    def test_index_create_view_no_permission(self):
        response = self._request_test_index_create_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(Index.objects.count(), 0)

    def test_index_create_view_with_permission(self):
        self.grant_permission(
            permission=permission_document_indexing_create
        )

        response = self._request_test_index_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Index.objects.count(), 1)
        self.assertEqual(Index.objects.first().label, TEST_INDEX_LABEL)

    def test_index_delete_view_no_permission(self):
        self._create_test_index()

        response = self._request_test_index_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Index.objects.count(), 1)

    def test_index_delete_view_with_permission(self):
        self._create_test_index()

        self.grant_permission(permission=permission_document_indexing_delete)

        response = self._request_test_index_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Index.objects.count(), 0)

    def test_index_document_types_view_no_permission(self):
        self._create_test_index()

        response = self._request_test_index_document_type_view()
        self.assertEqual(response.status_code, 404)

    def test_index_document_types_view_with_access(self):
        self._create_test_index()
        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_edit
        )

        response = self._request_test_index_document_type_view()
        self.assertEqual(response.status_code, 200)

    def test_index_edit_view_no_permission(self):
        self._create_test_index()

        response = self._request_test_index_edit_view()
        self.assertEqual(response.status_code, 404)

        self.test_index.refresh_from_db()
        self.assertEqual(self.test_index.label, TEST_INDEX_LABEL)

    def test_index_edit_view_with_access(self):
        self._create_test_index()

        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_edit
        )

        response = self._request_test_index_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_index.refresh_from_db()
        self.assertEqual(self.test_index.label, TEST_INDEX_LABEL_EDITED)


class IndexInstaceViewTestCase(
    IndexTestMixin, IndexViewTestMixin, IndexInstaceViewTestMixin,
    GenericDocumentViewTestCase
):
    def test_index_rebuild_view_no_permission(self):
        self._upload_test_document()
        self._create_test_index()
        self._create_test_index_template_node()

        response = self._request_test_index_rebuild_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(IndexInstanceNode.objects.count(), 1)
        self.assertEqual(IndexInstanceNode.objects.first().parent, None)

    def test_index_rebuild_view_with_access(self):
        self._upload_test_document()
        self._create_test_index()
        self._create_test_index_template_node()

        self.grant_access(
            obj=self.test_index,
            permission=permission_document_indexing_rebuild
        )

        response = self._request_test_index_rebuild_view()
        self.assertEqual(response.status_code, 302)

        self.assertNotEqual(IndexInstanceNode.objects.count(), 0)

    def test_index_instance_node_view_no_permission(self):
        self._create_test_index()

        response = self._request_test_index_instance_node_view(
            index_instance_node=self.test_index.instance_root
        )
        self.assertEqual(response.status_code, 403)

    def test_index_instance_node_view_with_access(self):
        self._create_test_index()

        self.grant_access(
            obj=self.test_index,
            permission=permission_document_indexing_instance_view
        )

        response = self._request_test_index_instance_node_view(
            index_instance_node=self.test_index.instance_root
        )
        self.assertContains(response, text=TEST_INDEX_LABEL, status_code=200)


class IndexTemplateNodeViewTestCase(
    IndexTestMixin, IndexTemplateNodeViewTestMixin, GenericViewTestCase
):
    def test_index_template_node_create_view_no_permission(self):
        self._create_test_index()
        node_count = self.test_index.node_templates.count()

        response = self._request_test_index_node_create_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(
            self.test_index.node_templates.count(), node_count
        )

    def test_index_template_node_create_view_with_access(self):
        self._create_test_index()
        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_edit
        )
        node_count = self.test_index.node_templates.count()

        response = self._request_test_index_node_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_index.node_templates.count(), node_count + 1
        )

    def test_index_template_node_delete_view_no_permission(self):
        self._create_test_index()
        self._create_test_index_template_node()
        node_count = self.test_index.node_templates.count()

        response = self._request_test_index_node_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self.test_index.node_templates.count(), node_count
        )

    def test_index_template_node_delete_view_with_access(self):
        self._create_test_index()
        self._create_test_index_template_node()
        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_edit
        )
        node_count = self.test_index.node_templates.count()

        response = self._request_test_index_node_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_index.node_templates.count(), node_count - 1
        )

    def test_index_template_node_edit_view_no_permission(self):
        self._create_test_index()
        self._create_test_index_template_node()
        node_expression = self.test_index_template_node.expression

        response = self._request_test_index_node_edit_view()
        self.assertEqual(response.status_code, 404)

        self.test_index_template_node.refresh_from_db()
        self.assertEqual(
            self.test_index_template_node.expression, node_expression
        )

    def test_index_template_node_edit_view_with_access(self):
        self._create_test_index()
        self._create_test_index_template_node()
        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_edit
        )
        node_expression = self.test_index_template_node.expression

        response = self._request_test_index_node_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_index_template_node.refresh_from_db()
        self.assertNotEqual(
            self.test_index_template_node.expression, node_expression
        )

    def test_index_template_node_list_get_view_no_permission(self):
        self._create_test_index()

        response = self._request_test_index_node_list_get_view()
        self.assertEqual(response.status_code, 404)

    def test_index_template_node_list_get_view_with_access(self):
        self._create_test_index()
        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_edit
        )

        response = self._request_test_index_node_list_get_view()
        self.assertEqual(response.status_code, 200)


class IndexToolsViewTestCase(
    IndexTestMixin, IndexViewTestMixin, IndexToolsViewTestMixin,
    GenericDocumentViewTestCase
):
    def test_indexes_rebuild_no_permission(self):
        self._create_test_index(rebuild=False)

        response = self._request_indexes_rebuild_get_view()
        self.assertNotContains(
            response=response, text=self.test_index.label, status_code=200
        )

        response = self._request_indexes_rebuild_post_view()
        # No error since we just don't see the index
        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            self.test_index.instance_root.get_children_count(), 0
        )

    def test_indexes_rebuild_with_access(self):
        self._create_test_index(rebuild=False)

        self.grant_access(
            obj=self.test_index,
            permission=permission_document_indexing_rebuild
        )

        response = self._request_indexes_rebuild_get_view()
        self.assertContains(
            response=response, text=self.test_index.label, status_code=200
        )

        response = self._request_indexes_rebuild_post_view()
        self.assertEqual(response.status_code, 302)

        # An instance root exists
        self.assertTrue(self.test_index.instance_root.pk)

    def test_indexes_reset_no_permission(self):
        self._create_test_index(rebuild=False)
        self._create_test_index_template_node()
        self.test_index.rebuild()

        response = self._request_indexes_reset_get_view()
        self.assertNotContains(
            response=response, text=self.test_index.label, status_code=200
        )

        response = self._request_indexes_reset_post_view()
        # No error since we just don't see the index
        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            self.test_index.instance_root.get_children_count(), 1
        )

    def test_indexes_reset_with_access(self):
        self._create_test_index(rebuild=False)
        self._create_test_index_template_node()
        self.test_index.rebuild()

        self.grant_access(
            obj=self.test_index,
            permission=permission_document_indexing_rebuild
        )

        response = self._request_indexes_reset_get_view()
        self.assertContains(
            response=response, text=self.test_index.label, status_code=200
        )

        response = self._request_indexes_reset_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_index.instance_root.get_children_count(), 0
        )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.queues import queue_tools

queue_tools.add_task_type(
    dotted_path='mayan.apps.file_caching.tasks.task_cache_purge',
    label=_('Purge a file cache')
)



import logging
import os
import sys
import traceback
import warnings

from django import apps
from django.conf import settings
from django.conf.urls import include, url
from django.contrib import admin
from django.contrib.auth.signals import user_logged_in
from django.db.models.signals import post_save
from django.utils.encoding import force_text
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _

from .classes import Template
from .handlers import (
    handler_pre_initial_setup, handler_pre_upgrade,
    handler_user_locale_profile_session_config, handler_user_locale_profile_create
)
from .links import (
    link_about, link_book, link_current_user_locale_profile_edit, link_license,
    link_object_error_list_clear, link_setup, link_store, link_support, link_tools
)

from .literals import MESSAGE_SQLITE_WARNING
from .menus import menu_about, menu_secondary, menu_topbar, menu_user
from .patches import patchDjangoTranslation
from .settings import (
    setting_auto_logging, setting_production_error_log_path,
    setting_production_error_logging, setting_url_base_path
)
from .signals import pre_initial_setup, pre_upgrade
from .tasks import task_delete_stale_uploads  # NOQA - Force task registration
from .utils import check_for_sqlite
from .warnings import DatabaseWarning

logger = logging.getLogger(name=__name__)


class MayanAppConfig(apps.AppConfig):
    app_namespace = None
    app_url = None

    def ready(self):
        logger.debug('Initializing app: %s', self.name)
        from mayan.urls import urlpatterns as mayan_urlpatterns

        installation_base_url = setting_url_base_path.value
        if installation_base_url:
            installation_base_url = '{}/'.format(installation_base_url)
        else:
            installation_base_url = ''

        if self.app_url:
            top_url = '{installation_base_url}{app_urls}/'.format(
                installation_base_url=installation_base_url,
                app_urls=self.app_url
            )
        elif self.app_url is not None:
            # When using app_url as '' to register a top of URL view.
            top_url = installation_base_url
        else:
            # If app_url is None, use the app's name for the URL base.
            top_url = '{installation_base_url}{app_name}/'.format(
                installation_base_url=installation_base_url,
                app_name=self.name
            )

        try:
            app_urlpatterns = import_string(
                dotted_path='{}.urls.urlpatterns'.format(self.name)
            )
        except ImportError as exception:
            non_critical_error_list = (
                'No module named urls',
                'No module named \'{}.urls\''.format(self.name),
                'Module "{}.urls" does not define a "urlpatterns" attribute/class'.format(self.name),
            )
            if force_text(exception) not in non_critical_error_list:
                logger.exception(
                    'Import time error when running AppConfig.ready() of app '
                    '"%s".', self.name
                )
                exc_info = sys.exc_info()
                traceback.print_exception(*exc_info)
                raise exception
        else:
            mayan_urlpatterns += (
                url(
                    regex=r'^{}'.format(top_url), view=include(
                        (app_urlpatterns, self.app_namespace or self.name)
                    )
                ),
            )

        try:
            passthru_urlpatterns = import_string(
                dotted_path='{}.urls.passthru_urlpatterns'.format(self.name)
            )
        except ImportError as exception:
            non_critical_error_list = (
                'No module named urls',
                'No module named \'{}.urls\''.format(self.name),
                'Module "{}.urls" does not define a "passthru_urlpatterns" attribute/class'.format(self.name),
            )
            if force_text(exception) not in non_critical_error_list:
                logger.exception(
                    'Import time error when running AppConfig.ready() of app '
                    '"%s".', self.name
                )
                exc_info = sys.exc_info()
                traceback.print_exception(*exc_info)
                raise exception
        else:
            mayan_urlpatterns += (
                url(
                    regex=r'^{}'.format(top_url), view=include(
                        (passthru_urlpatterns)
                    )
                ),
            )


class CommonApp(MayanAppConfig):
    app_namespace = 'common'
    app_url = ''
    has_rest_api = True
    has_tests = True
    name = 'mayan.apps.common'
    verbose_name = _('Common')

    def ready(self):
        super(CommonApp, self).ready()
        patchDjangoTranslation()

        admin.autodiscover()

        if check_for_sqlite():
            warnings.warn(
                category=DatabaseWarning,
                message=force_text(MESSAGE_SQLITE_WARNING)
            )

        Template(
            name='menu_main', template_name='appearance/menu_main.html'
        )
        Template(
            name='menu_topbar', template_name='appearance/menu_topbar.html'
        )

        menu_user.bind_links(
            links=(
                link_current_user_locale_profile_edit,
            ), position=50
        )

        menu_about.bind_links(
            links=(
                link_tools, link_setup, link_about, link_book, link_store,
                link_support, link_license,
            )
        )

        menu_topbar.bind_links(links=(menu_about, menu_user,), position=99)
        menu_secondary.bind_links(
            links=(link_object_error_list_clear,), sources=(
                'common:object_error_list',
            )
        )

        post_save.connect(
            dispatch_uid='common_handler_user_locale_profile_create',
            receiver=handler_user_locale_profile_create,
            sender=settings.AUTH_USER_MODEL
        )
        pre_initial_setup.connect(
            dispatch_uid='common_handler_pre_initial_setup',
            receiver=handler_pre_initial_setup
        )
        pre_upgrade.connect(
            dispatch_uid='common_handler_pre_upgrade',
            receiver=handler_pre_upgrade
        )

        user_logged_in.connect(
            dispatch_uid='common_handler_user_locale_profile_session_config',
            receiver=handler_user_locale_profile_session_config
        )
        self.setup_auto_logging()

    def setup_auto_logging(self):
        if setting_auto_logging.value:
            if settings.DEBUG:
                level = 'DEBUG'
                handlers = ['console']
            else:
                level = 'ERROR'
                handlers = ['console']

            if os.path.exists(settings.MEDIA_ROOT) and setting_production_error_logging.value:
                handlers.append('logfile')

            loggers = {}
            for project_app in apps.apps.get_app_configs():
                loggers[project_app.name] = {
                    'handlers': handlers,
                    'propagate': True,
                    'level': level,
                }

            logging_configuration = {
                'version': 1,
                'disable_existing_loggers': False,
                'formatters': {
                    'intermediate': {
                        '()': 'mayan.apps.common.log.ColorFormatter',
                        'format': '%(name)s <%(process)d> [%(levelname)s] "%(funcName)s() line %(lineno)d %(message)s"',
                    },
                    'logfile': {
                        'format': '%(asctime)s %(name)s <%(process)d> [%(levelname)s] "%(funcName)s() line %(lineno)d %(message)s"'
                    },
                },
                'handlers': {
                    'console': {
                        'class': 'logging.StreamHandler',
                        'formatter': 'intermediate',
                        'level': 'DEBUG',
                    },
                },
                'loggers': loggers
            }

            if os.path.exists(settings.MEDIA_ROOT) and setting_production_error_logging.value:
                logging_configuration['handlers']['logfile'] = {
                    'backupCount': 5,
                    'class': 'logging.handlers.RotatingFileHandler',
                    'filename': setting_production_error_log_path.value,
                    'formatter': 'logfile',
                    'maxBytes': 65535,
                }

            logging.config.dictConfig(logging_configuration)



from django.contrib import messages
from django.http import HttpResponseRedirect
from django.urls import reverse, reverse_lazy
from django.utils.translation import ugettext_lazy as _, ungettext

from mayan.apps.common.generics import (
    FormView, MultipleObjectConfirmActionView, SingleObjectDetailView,
    SingleObjectDownloadView, SingleObjectEditView, SingleObjectListView
)
from mayan.apps.common.mixins import ExternalObjectMixin
from mayan.apps.documents.forms import DocumentTypeFilteredSelectForm
from mayan.apps.documents.models import Document, DocumentPage, DocumentType

from .forms import DocumentPageOCRContentForm, DocumentOCRContentForm
from .models import DocumentPageOCRContent, DocumentVersionOCRError
from .permissions import (
    permission_ocr_content_view, permission_ocr_document,
    permission_document_type_ocr_setup
)
from .utils import get_instance_ocr_content


class DocumentOCRContentDeleteView(MultipleObjectConfirmActionView):
    model = Document
    object_permission = permission_ocr_document
    pk_url_kwarg = 'document_id'
    success_message = 'Deleted OCR content of %(count)d document.'
    success_message_plural = 'Deleted OCR content of %(count)d documents.'

    def get_extra_context(self):
        queryset = self.object_list

        result = {
            'title': ungettext(
                singular='Delete the OCR content of the selected document?',
                plural='Delete the OCR content of the selected documents?',
                number=queryset.count()
            )
        }

        if queryset.count() == 1:
            result['object'] = queryset.first()

        return result

    def object_action(self, form, instance):
        DocumentPageOCRContent.objects.delete_content_for(
            document=instance, user=self.request.user
        )


class DocumentOCRContentView(SingleObjectDetailView):
    form_class = DocumentOCRContentForm
    model = Document
    object_permission = permission_ocr_content_view
    pk_url_kwarg = 'document_id'

    def dispatch(self, request, *args, **kwargs):
        result = super(DocumentOCRContentView, self).dispatch(
            request, *args, **kwargs
        )
        self.object.add_as_recent_document_for_user(user=request.user)
        return result

    def get_extra_context(self):
        return {
            'document': self.object,
            'hide_labels': True,
            'object': self.object,
            'title': _('OCR result for document: %s') % self.object,
        }


class DocumentOCRDownloadView(SingleObjectDownloadView):
    model = Document
    object_permission = permission_ocr_content_view
    pk_url_kwarg = 'document_id'

    def get_download_file_object(self):
        return get_instance_ocr_content(instance=self.object)

    def get_download_filename(self):
        return '{}-OCR'.format(self.object)


class DocumentOCRErrorsListView(ExternalObjectMixin, SingleObjectListView):
    external_object_class = Document
    external_object_permission = permission_ocr_document
    external_object_pk_url_kwarg = 'document_id'

    def get_extra_context(self):
        return {
            'hide_object': True,
            'object': self.external_object,
            'title': _('OCR errors for document: %s') % self.external_object,
        }

    def get_source_queryset(self):
        return self.external_object.latest_version.ocr_errors.all()


class DocumentPageOCRContentView(SingleObjectDetailView):
    form_class = DocumentPageOCRContentForm
    model = DocumentPage
    object_permission = permission_ocr_content_view
    pk_url_kwarg = 'document_page_id'

    def dispatch(self, request, *args, **kwargs):
        result = super(DocumentPageOCRContentView, self).dispatch(
            request, *args, **kwargs
        )
        self.object.document.add_as_recent_document_for_user(
            user=request.user
        )
        return result

    def get_extra_context(self):
        return {
            'hide_labels': True,
            'object': self.object,
            'title': _('OCR result for document page: %s') % self.object,
        }


class DocumentSubmitView(MultipleObjectConfirmActionView):
    model = Document
    object_permission = permission_ocr_document
    pk_url_kwarg = 'document_id'
    success_message = '%(count)d document submitted to the OCR queue.'
    success_message_plural = '%(count)d documents submitted to the OCR queue.'

    def get_extra_context(self):
        queryset = self.object_list

        result = {
            'title': ungettext(
                singular='Submit the selected document to the OCR queue?',
                plural='Submit the selected documents to the OCR queue?',
                number=queryset.count()
            )
        }

        if queryset.count() == 1:
            result['object'] = queryset.first()

        return result

    def object_action(self, form, instance):
        instance.submit_for_ocr()


class DocumentTypeSubmitView(FormView):
    extra_context = {
        'title': _('Submit all documents of a type for OCR')
    }
    form_class = DocumentTypeFilteredSelectForm
    post_action_redirect = reverse_lazy(viewname='common:tools_list')

    def form_valid(self, form):
        count = 0
        for document_type in form.cleaned_data['document_type']:
            for document in document_type.documents.all():
                document.submit_for_ocr()
                count += 1

        messages.success(
            message=_(
                '%(count)d documents added to the OCR queue.'
            ) % {
                'count': count,
            }, request=self.request
        )

        return HttpResponseRedirect(redirect_to=self.get_success_url())

    def get_form_extra_kwargs(self):
        return {
            'allow_multiple': True,
            'permission': permission_ocr_document,
            'user': self.request.user
        }

    def get_post_action_redirect(self):
        return reverse(viewname='common:tools_list')


class DocumentTypeSettingsEditView(ExternalObjectMixin, SingleObjectEditView):
    external_object_class = DocumentType
    external_object_permission = permission_document_type_ocr_setup
    external_object_pk_url_kwarg = 'document_type_id'
    fields = ('auto_ocr',)
    post_action_redirect = reverse_lazy(
        viewname='documents:document_type_list'
    )

    def get_document_type(self):
        return self.external_object

    def get_extra_context(self):
        return {
            'object': self.get_document_type(),
            'title': _(
                'Edit OCR settings for document type: %s.'
            ) % self.get_document_type()
        }

    def get_object(self, queryset=None):
        return self.get_document_type().ocr_settings


class EntryListView(SingleObjectListView):
    extra_context = {
        'hide_object': True,
        'title': _('OCR errors'),
    }
    view_permission = permission_document_type_ocr_setup

    def get_source_queryset(self):
        return DocumentVersionOCRError.objects.all()



from django.contrib.contenttypes.models import ContentType

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.permissions.tests.mixins import PermissionTestMixin

from ..classes import Layer
from ..models import ObjectLayer
from ..transformations import BaseTransformation

from .literals import (
    TEST_TRANSFORMATION_CLASS_LABEL, TEST_TRANSFORMATION_CLASS_NAME,
    TEST_TRANSFORMATION_NAME, TEST_TRANSFORMATION_ARGUMENT,
    TEST_TRANSFORMATION_ARGUMENT_EDITED
)


class LayerTestCaseMixin(object):
    def setUp(self):
        super(LayerTestCaseMixin, self).setUp()
        Layer.invalidate_cache()


class LayerTestMixin(PermissionTestMixin):
    test_layer = Layer(
        label='Test layer', name='test_layer', order=1000,
        permissions={}
    )

    def setUp(self):
        super(LayerTestMixin, self).setUp()
        self._create_test_permission()

        self.test_layer_permission = self.test_permission
        ModelPermission.register(
            model=self.test_document._meta.model, permissions=(
                self.test_permission,
            )
        )

        self.test_layer.permissions = {
            'create': self.test_layer_permission,
            'delete': self.test_layer_permission,
            'edit': self.test_layer_permission,
            'select': self.test_layer_permission,
            'view': self.test_layer_permission,
        }


class TransformationTestMixin(LayerTestMixin):
    def _create_test_transformation(self):
        content_type = ContentType.objects.get_for_model(
            model=self.test_document
        )
        object_layer, created = ObjectLayer.objects.get_or_create(
            content_type=content_type, object_id=self.test_document.pk,
            stored_layer=self.test_layer.stored_layer
        )

        self.test_transformation = object_layer.transformations.create(
            name=TEST_TRANSFORMATION_NAME,
            arguments=TEST_TRANSFORMATION_ARGUMENT
        )

    def _create_test_transformation_class(self):
        class TestTransformation(BaseTransformation):
            arguments = ('test_argument',)
            label = TEST_TRANSFORMATION_CLASS_LABEL
            name = TEST_TRANSFORMATION_CLASS_NAME

            def execute_on(self, *args, **kwargs):
                super(TestTransformation, self).execute_on(*args, **kwargs)

                return self.image

        BaseTransformation.register(
            layer=self.test_layer, transformation=TestTransformation
        )

        self.TestTransformationClass = TestTransformation


class TransformationViewsTestMixin(object):
    def _request_transformation_create_view(self):
        return self.post(
            viewname='converter:transformation_create', kwargs={
                'app_label': 'documents', 'model_name': 'document',
                'object_id': self.test_document.pk,
                'layer_name': self.test_layer.name,
                'transformation_name': TEST_TRANSFORMATION_NAME,
            }, data={
                'arguments': TEST_TRANSFORMATION_ARGUMENT
            }
        )

    def _request_transformation_delete_view(self):
        return self.post(
            viewname='converter:transformation_delete', kwargs={
                'layer_name': self.test_layer.name,
                'transformation_id': self.test_transformation.pk
            }
        )

    def _request_transformation_edit_view(self):
        return self.post(
            viewname='converter:transformation_edit', kwargs={
                'layer_name': self.test_layer.name,
                'transformation_id': self.test_transformation.pk
            }, data={
                'arguments': TEST_TRANSFORMATION_ARGUMENT_EDITED
            }
        )

    def _request_transformation_list_view(self):
        return self.get(
            viewname='converter:transformation_list', kwargs={
                'app_label': 'documents', 'model_name': 'document',
                'object_id': self.test_document.pk,
                'layer_name': self.test_layer.name
            }
        )

    def _request_test_transformation_select_get_view(self):
        return self.get(
            viewname='converter:transformation_select', kwargs={
                'app_label': 'documents', 'model_name': 'document',
                'object_id': self.test_document.pk,
                'layer_name': self.test_layer.name
            }
        )

    def _request_test_transformation_select_post_view(self):
        return self.post(
            viewname='converter:transformation_select', kwargs={
                'app_label': 'documents', 'model_name': 'document',
                'object_id': self.test_document.pk,
                'layer_name': self.test_layer.name
            }, data={
                'transformation': TEST_TRANSFORMATION_CLASS_NAME
            }
        )



from django.apps import apps

from .events import event_user_logged_in, event_user_logged_out


def handler_initialize_new_user_options(sender, instance, **kwargs):
    UserOptions = apps.get_model(
        app_label='user_management', model_name='UserOptions'
    )

    if kwargs['created']:
        UserOptions.objects.create(user=instance)


def handler_user_logged_in(sender, **kwargs):
    event_user_logged_in.commit(
        actor=kwargs['user'], target=kwargs['user']
    )


def handler_user_logged_out(sender, **kwargs):
    event_user_logged_out.commit(
        actor=kwargs['user'], target=kwargs['user']
    )



from django import forms
from django.utils.translation import ugettext_lazy as _

from ..fields import DocumentPageField

__all__ = ('DocumentPageForm', 'DocumentPageNumberForm')


class DocumentPageForm(forms.Form):
    def __init__(self, *args, **kwargs):
        instance = kwargs.pop('instance', None)
        rotation = kwargs.pop('rotation', None)
        zoom = kwargs.pop('zoom', None)
        super(DocumentPageForm, self).__init__(*args, **kwargs)
        self.fields['document_page'].initial = instance
        self.fields['document_page'].widget.attrs.update({
            'zoom': zoom,
            'rotation': rotation,
        })

    document_page = DocumentPageField()


class DocumentPageNumberForm(forms.Form):
    page = forms.ModelChoiceField(
        help_text=_(
            'Page number from which all the transformations will be cloned. '
            'Existing transformations will be lost.'
        ), queryset=None
    )

    def __init__(self, *args, **kwargs):
        self.document = kwargs.pop('document')
        super(DocumentPageNumberForm, self).__init__(*args, **kwargs)
        self.fields['page'].queryset = self.document.pages.all()



from django.urls import reverse
from django.utils.translation import ugettext_lazy as _, ungettext

from mayan.apps.acls.models import AccessControlList
from mayan.apps.common.exceptions import ActionError
from mayan.apps.common.generics import (
    MultipleObjectConfirmActionView, MultipleObjectFormActionView,
    SingleObjectDetailView
)
from mayan.apps.documents.models import Document
from mayan.apps.documents.views.document_views import DocumentListView

from .exceptions import DocumentAlreadyCheckedOut, DocumentNotCheckedOut
from .forms import DocumentCheckOutForm, DocumentCheckOutDetailForm
from .icons import icon_check_out_info
from .models import DocumentCheckout
from .permissions import (
    permission_document_check_in, permission_document_check_in_override,
    permission_document_check_out, permission_document_check_out_detail_view
)


class DocumentCheckInView(MultipleObjectConfirmActionView):
    error_message = 'Unable to check in document "%(instance)s". %(exception)s'
    model = Document
    pk_url_kwarg = 'document_id'
    success_message_singular = '%(count)d document checked in.'
    success_message_plural = '%(count)d documents checked in.'

    def get_extra_context(self):
        queryset = self.get_object_list()

        result = {
            'title': ungettext(
                singular='Check in %(count)d document',
                plural='Check in %(count)d documents',
                number=queryset.count()
            ) % {
                'count': queryset.count(),
            }
        }

        if queryset.count() == 1:
            result.update(
                {
                    'object': queryset.first(),
                    'title': _(
                        'Check in document: %s'
                    ) % queryset.first()
                }
            )

        return result

    def get_post_object_action_url(self):
        if self.action_count == 1:
            return reverse(
                viewname='checkouts:check_out_info', kwargs={
                    'document_id': self.action_id_list[0]
                }
            )
        else:
            super(DocumentCheckInView, self).get_post_action_redirect()

    def get_source_queryset(self):
        # object_permission is None to disable restricting queryset mixin
        # and restrict the queryset ourselves from two permissions

        source_queryset = super(DocumentCheckInView, self).get_source_queryset()

        check_in_queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_document_check_in, queryset=source_queryset,
            user=self.request.user
        )

        check_in_override_queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_document_check_in_override,
            queryset=source_queryset, user=self.request.user
        )

        return check_in_queryset | check_in_override_queryset

    def object_action(self, form, instance):
        try:
            DocumentCheckout.business_logic.check_in_document(
                document=instance, user=self.request.user
            )
        except DocumentNotCheckedOut as exception:
            raise ActionError(exception)


class DocumentCheckOutView(MultipleObjectFormActionView):
    error_message = 'Unable to checkout document "%(instance)s". %(exception)s'
    form_class = DocumentCheckOutForm
    model = Document
    object_permission = permission_document_check_out
    pk_url_kwarg = 'document_id'
    success_message_singular = '%(count)d document checked out.'
    success_message_plural = '%(count)d documents checked out.'

    def get_extra_context(self):
        queryset = self.get_object_list()

        result = {
            'title': ungettext(
                singular='Checkout %(count)d document',
                plural='Checkout %(count)d documents',
                number=queryset.count()
            ) % {
                'count': queryset.count(),
            }
        }

        if queryset.count() == 1:
            result.update(
                {
                    'object': queryset.first(),
                    'title': _(
                        'Check out document: %s'
                    ) % queryset.first()
                }
            )

        return result

    def get_post_object_action_url(self):
        if self.action_count == 1:
            return reverse(
                viewname='checkouts:check_out_info', kwargs={
                    'document_id': self.action_id_list[0]
                }
            )
        else:
            super(DocumentCheckOutView, self).get_post_action_redirect()

    def object_action(self, form, instance):
        try:
            DocumentCheckout.objects.check_out_document(
                block_new_version=form.cleaned_data['block_new_version'],
                document=instance,
                expiration_datetime=form.cleaned_data['expiration_datetime'],
                user=self.request.user,
            )
        except DocumentAlreadyCheckedOut as exception:
            raise ActionError(exception)


class DocumentCheckOutDetailView(SingleObjectDetailView):
    form_class = DocumentCheckOutDetailForm
    model = Document
    object_permission = permission_document_check_out_detail_view
    pk_url_kwarg = 'document_id'

    def get_extra_context(self):
        return {
            'object': self.object,
            'title': _(
                'Check out details for document: %s'
            ) % self.object
        }


class DocumentCheckOutListView(DocumentListView):
    def get_document_queryset(self):
        return AccessControlList.objects.restrict_queryset(
            permission=permission_document_check_out_detail_view,
            queryset=DocumentCheckout.objects.checked_out_documents(),
            user=self.request.user
        )

    def get_extra_context(self):
        context = super(DocumentCheckOutListView, self).get_extra_context()
        context.update(
            {
                'no_results_icon': icon_check_out_info,
                'no_results_text': _(
                    'Checking out a document, blocks certain operations '
                    'for a predetermined amount of time.'
                ),
                'no_results_title': _('No documents have been checked out'),
                'title': _('Checked out documents'),
            }
        )
        return context



from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig

from .classes import DefinedStorage


class StorageApp(MayanAppConfig):
    has_tests = True
    name = 'mayan.apps.storage'
    verbose_name = _('Storage')

    def ready(self):
        super(StorageApp, self).ready()
        DefinedStorage.initialize()



from mayan.apps.appearance.classes import Icon

icon_document_submit = Icon(
    driver_name='fontawesome-dual', primary_symbol='chess-board',
    secondary_symbol='arrow-right'
)
icon_document_multiple_submit = Icon(
    driver_name='fontawesome-dual', primary_symbol='chess-board',
    secondary_symbol='arrow-right'
)
icon_document_type_submit = Icon(
    driver_name='fontawesome-dual', primary_symbol='chess-board',
    secondary_symbol='arrow-right'
)
icon_file_metadata = Icon(
    driver_name='fontawesome', symbol='chess-board'
)



from django.test import override_settings

from mayan.apps.documents.tests.base import GenericDocumentTestCase
from mayan.apps.documents.tests.literals import TEST_HYBRID_DOCUMENT

from .literals import TEST_DOCUMENT_CONTENT


class DocumentAutoParsingTestCase(GenericDocumentTestCase):
    test_document_filename = TEST_HYBRID_DOCUMENT
    auto_create_test_document_type = False

    def test_disable_auto_parsing(self):
        self._create_test_document_type()
        self._upload_test_document()
        with self.assertRaises(expected_exception=StopIteration):
            next(self.test_document.latest_version.content())

    @override_settings(DOCUMENT_PARSING_AUTO_PARSING=True)
    def test_enabled_auto_parsing(self):
        self._create_test_document_type()
        self._upload_test_document()
        self.assertTrue(
            TEST_DOCUMENT_CONTENT in next(self.test_document.content())
        )



from django.contrib import admin

from .models import (
    IMAPEmail, POP3Email, StagingFolderSource, WatchFolderSource, WebFormSource
)


@admin.register(IMAPEmail)
class IMAPEmailAdmin(admin.ModelAdmin):
    list_display = (
        'label', 'enabled', 'uncompress', 'host', 'ssl', 'port', 'interval',
        'document_type', 'metadata_attachment_name', 'from_metadata_type',
        'subject_metadata_type', 'store_body'
    )


@admin.register(POP3Email)
class POP3EmailAdmin(admin.ModelAdmin):
    list_display = (
        'label', 'enabled', 'uncompress', 'host', 'ssl', 'port', 'interval',
        'document_type', 'metadata_attachment_name', 'from_metadata_type',
        'subject_metadata_type', 'store_body'
    )


@admin.register(StagingFolderSource)
class StagingFolderSourceAdmin(admin.ModelAdmin):
    list_display = (
        'label', 'enabled', 'folder_path', 'preview_width', 'preview_height',
        'uncompress', 'delete_after_upload'
    )


@admin.register(WatchFolderSource)
class WatchFolderSourceAdmin(admin.ModelAdmin):
    list_display = ('label', 'enabled', 'folder_path', 'uncompress')


@admin.register(WebFormSource)
class WebFormSourceAdmin(admin.ModelAdmin):
    list_display = ('label', 'enabled', 'uncompress')



from mayan.apps.common.tests.base import GenericViewTestCase

from ..permissions import permission_settings_view

from .mixins import SmartSettingTestMixin, SmartSettingViewTestMixin


class SmartSettingViewTestCase(
    SmartSettingTestMixin, SmartSettingViewTestMixin, GenericViewTestCase
):
    def setUp(self):
        super(SmartSettingViewTestCase, self).setUp()
        self._create_test_settings_namespace()

    def test_namespace_list_view_no_permission(self):
        response = self._request_namespace_list_view()
        self.assertEqual(response.status_code, 403)

    def test_namespace_detail_view_no_permission(self):
        response = self._request_namespace_detail_view()
        self.assertEqual(response.status_code, 403)

    def test_namespace_list_view_with_permission(self):
        self.grant_permission(permission=permission_settings_view)
        response = self._request_namespace_list_view()
        self.assertEqual(response.status_code, 200)

    def test_namespace_detail_view_with_permission(self):
        self.grant_permission(permission=permission_settings_view)
        response = self._request_namespace_detail_view()
        self.assertEqual(response.status_code, 200)



from django.apps import apps
from django.db.models.signals import pre_delete, pre_save
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig

from .handlers import handler_document_cache_delete, handler_node_cache_delete


class MirroringApp(MayanAppConfig):
    has_tests = True
    name = 'mayan.apps.mirroring'
    verbose_name = _('Mirroring')

    def ready(self):
        super(MirroringApp, self).ready()

        Document = apps.get_model(app_label='documents', model_name='Document')
        IndexInstanceNode = apps.get_model(
            app_label='document_indexing', model_name='IndexInstanceNode'
        )

        pre_delete.connect(
            handler_document_cache_delete,
            dispatch_uid='mirroring_handler_document_cache_delete',
            sender=Document
        )
        pre_delete.connect(
            handler_node_cache_delete,
            dispatch_uid='mirroring_handler_node_cache_delete',
            sender=IndexInstanceNode
        )
        pre_save.connect(
            handler_document_cache_delete,
            dispatch_uid='mirroring_handler_document_cache_delete',
            sender=Document
        )
        pre_save.connect(
            handler_node_cache_delete,
            dispatch_uid='mirroring_handler_node_cache_delete',
            sender=IndexInstanceNode
        )



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('linking', '0009_auto_20191211_0233'),
    ]

    operations = [
        migrations.AlterField(
            model_name='smartlink',
            name='label',
            field=models.CharField(
                db_index=True, help_text='A short text describing the '
                'smart link.', max_length=128, verbose_name='Label'
            ),
        ),
    ]



from ..models import WebLink

from .literals import (
    TEST_WEB_LINK_LABEL, TEST_WEB_LINK_LABEL_EDITED, TEST_WEB_LINK_TEMPLATE
)


class ResolvedWebLinkAPIViewTestMixin(object):
    def _request_resolved_web_link_detail_view(self):
        return self.get(
            viewname='rest_api:resolved_web_link-detail',
            kwargs={
                'pk': self.test_document.pk,
                'resolved_web_link_pk': self.test_web_link.pk
            }
        )
        self._create_test_web_link(add_test_document_type=True)

    def _request_resolved_web_link_list_view(self):
        return self.get(
            viewname='rest_api:resolved_web_link-list', kwargs={
                'pk': self.test_document.pk
            }
        )

    def _request_resolved_web_link_navigate_view(self):
        return self.get(
            viewname='rest_api:resolved_web_link-navigate',
            kwargs={
                'pk': self.test_document.pk,
                'resolved_web_link_pk': self.test_web_link.pk
            }
        )


class WebLinkAPIViewTestMixin(object):
    def _request_test_web_link_create_api_view(self):
        return self.post(
            viewname='rest_api:web_link-list', data={
                'label': TEST_WEB_LINK_LABEL,
                'template': TEST_WEB_LINK_TEMPLATE
            }
        )

    def _request_test_web_link_create_with_document_type_api_view(self):
        return self.post(
            viewname='rest_api:web_link-list', data={
                'label': TEST_WEB_LINK_LABEL,
                'document_types_pk_list': self.test_document_type.pk,
                'template': TEST_WEB_LINK_TEMPLATE
            },
        )

    def _request_test_web_link_delete_api_view(self):
        return self.delete(
            viewname='rest_api:web_link-detail', kwargs={
                'pk': self.test_web_link.pk
            }
        )

    def _request_test_web_link_detail_api_view(self):
        return self.get(
            viewname='rest_api:web_link-detail', kwargs={
                'pk': self.test_web_link.pk
            }
        )

    def _request_test_web_link_edit_patch_api_view(self):
        return self.patch(
            viewname='rest_api:web_link-detail',
            kwargs={'pk': self.test_web_link.pk}, data={
                'label': TEST_WEB_LINK_LABEL_EDITED,
                'document_types_pk_list': self.test_document_type.pk
            }
        )

    def _request_test_web_link_edit_put_api_view(self):
        return self.put(
            viewname='rest_api:web_link-detail',
            kwargs={'pk': self.test_web_link.pk}, data={
                'label': TEST_WEB_LINK_LABEL_EDITED,
                'document_types_pk_list': self.test_document_type.pk,
                'template': TEST_WEB_LINK_TEMPLATE
            }
        )


class WebLinkTestMixin(object):
    def _create_test_web_link(self, add_document_type=False):
        self.test_web_link = WebLink.objects.create(
            label=TEST_WEB_LINK_LABEL, template=TEST_WEB_LINK_TEMPLATE,
        )
        if add_document_type:
            self.test_web_link.document_types.add(self.test_document_type)


class WebLinkViewTestMixin(object):
    def _request_test_document_web_link_instance_view(self):
        return self.post(
            viewname='web_links:web_link_instance_view', kwargs={
                'document_id': self.test_document.pk,
                'web_link_id': self.test_web_link.pk
            }
        )

    def _request_test_document_web_link_list_view(self):
        return self.get(
            viewname='web_links:document_web_link_list', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_test_web_link_create_view(self):
        return self.post(
            viewname='web_links:web_link_create', data={
                'label': TEST_WEB_LINK_LABEL,
                'template_template': TEST_WEB_LINK_TEMPLATE,
            }
        )

    def _request_test_web_link_delete_view(self):
        return self.post(
            viewname='web_links:web_link_delete', kwargs={
                'web_link_id': self.test_web_link.pk
            }
        )

    def _request_test_web_link_edit_view(self):
        return self.post(
            viewname='web_links:web_link_edit', kwargs={
                'web_link_id': self.test_web_link.pk
            }, data={
                'label': TEST_WEB_LINK_LABEL_EDITED,
                'template_template': TEST_WEB_LINK_TEMPLATE
            }
        )

    def _request_test_web_link_list_view(self):
        return self.get(viewname='web_links:web_link_list')



from django.apps import apps


def hook_is_new_version_allowed(document_version):
    NewVersionBlock = apps.get_model(
        app_label='checkouts', model_name='NewVersionBlock'
    )

    NewVersionBlock.objects.new_versions_allowed(
        document=document_version.document
    )



import os

from django.conf import settings
from django.utils.encoding import force_bytes

from mayan.apps.common.tests.mixins import EnvironmentTestCaseMixin
from mayan.apps.storage.utils import fs_cleanup, NamedTemporaryFile

from ..classes import Namespace, Setting
from ..utils import BaseSetting, SettingNamespaceSingleton

from .literals import (
    TEST_BOOTSTAP_SETTING_NAME, TEST_NAMESPACE_LABEL, TEST_NAMESPACE_NAME,
    TEST_SETTING_DEFAULT_VALUE, TEST_SETTING_GLOBAL_NAME
)


class BoostrapSettingTestMixin(object):
    def _create_test_bootstrap_singleton(self):
        self.test_globals = {}
        self.test_globals['BASE_DIR'] = ''
        self.test_setting_namespace_singleton = SettingNamespaceSingleton(
            global_symbol_table=self.test_globals
        )

    def _register_test_boostrap_setting(self):
        SettingNamespaceSingleton.register_setting(
            name=TEST_BOOTSTAP_SETTING_NAME, klass=BaseSetting, kwargs={
                'has_default': True, 'default_value': 'value default'
            }
        )


class SmartSettingsTestCaseMixin(object):
    def setUp(self):
        super(SmartSettingsTestCaseMixin, self).setUp()
        Namespace.invalidate_cache_all()

        with NamedTemporaryFile(delete=False) as self.test_setting_config_file_object:
            settings.CONFIGURATION_FILEPATH = self.test_setting_config_file_object.name
            os.environ['MAYAN_CONFIGURATION_FILEPATH'] = self.test_setting_config_file_object.name
            Setting._config_file_cache = None

    def tearDown(self):
        fs_cleanup(filename=self.test_setting_config_file_object.name)
        Namespace.invalidate_cache_all()
        super(SmartSettingsTestCaseMixin, self).tearDown()


class SmartSettingTestMixin(EnvironmentTestCaseMixin):
    test_setting_global_name = None
    test_config_file_object = None

    def tearDown(self):
        if self.test_config_file_object:
            fs_cleanup(filename=self.test_config_file_object.name)
        super(SmartSettingTestMixin, self).tearDown()

    def _create_test_config_file(self, callback=None):
        if not self.test_setting_global_name:
            self.test_setting_global_name = self.test_setting.global_name

        test_config_entry = {
            self.test_setting_global_name: self.test_config_value
        }

        with NamedTemporaryFile(delete=False) as test_config_file_object:
            # Needed to load the config file from the Setting class
            # after bootstrap.
            settings.CONFIGURATION_FILEPATH = test_config_file_object.name
            # Needed to update the globals before Mayan has loaded.
            self._set_environment_variable(
                name='MAYAN_CONFIGURATION_FILEPATH',
                value=test_config_file_object.name
            )
            test_config_file_object.write(
                force_bytes(
                    Setting.serialize_value(value=test_config_entry)
                )
            )
            test_config_file_object.seek(0)
            Setting._config_file_cache = None

            if callback:
                callback()

    def _create_test_settings_namespace(self, **kwargs):
        try:
            self.test_settings_namespace = Namespace.get(
                name=TEST_NAMESPACE_NAME
            )
            self.test_settings_namespace.migration_class = None
            self.test_settings_namespace.version = None
            self.test_settings_namespace.__dict__.update(kwargs)
        except KeyError:
            self.test_settings_namespace = Namespace(
                label=TEST_NAMESPACE_LABEL, name=TEST_NAMESPACE_NAME,
                **kwargs
            )

    def _create_test_setting(self):
        self.test_setting = self.test_settings_namespace.add_setting(
            global_name=TEST_SETTING_GLOBAL_NAME,
            default=TEST_SETTING_DEFAULT_VALUE
        )


class SmartSettingViewTestMixin(object):
    def _request_namespace_list_view(self):
        return self.get(viewname='settings:namespace_list')

    def _request_namespace_detail_view(self):
        return self.get(
            viewname='settings:namespace_detail', kwargs={
                'namespace_name': self.test_settings_namespace.name
            }
        )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.events.classes import EventTypeNamespace

namespace = EventTypeNamespace(
    label=_('Access control lists'), name='acls'
)

event_acl_created = namespace.add_event_type(
    label=_('ACL created'), name='acl_created'
)
event_acl_edited = namespace.add_event_type(
    label=_('ACL edited'), name='acl_edited'
)



from django.utils.module_loading import import_string

from ..permissions import permission_document_view
from ..search import document_search, document_page_search

from .base import GenericDocumentViewTestCase


class DocumentSearchTestMixin(object):
    search_backend = import_string(
        dotted_path='mayan.apps.dynamic_search.backends.django.DjangoSearchBackend'
    )()

    def _perform_document_page_search(self):
        return self.search_backend.search(
            search_model=document_page_search, query_string={'q': self.test_document.label},
            user=self._test_case_user
        )

    def _perform_document_search(self):
        return self.search_backend.search(
            search_model=document_search, query_string={'q': self.test_document.label},
            user=self._test_case_user
        )


class DocumentSearchTestCase(
    DocumentSearchTestMixin, GenericDocumentViewTestCase
):
    def test_document_page_search_no_access(self):
        queryset = self._perform_document_page_search()
        self.assertFalse(self.test_document.pages.first() in queryset)

    def test_document_page_search_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )
        queryset = self._perform_document_page_search()
        self.assertTrue(self.test_document.pages.first() in queryset)

    def test_document_search_no_access(self):
        queryset = self._perform_document_search()
        self.assertFalse(self.test_document in queryset)

    def test_document_search_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )
        queryset = self._perform_document_search()
        self.assertTrue(self.test_document in queryset)



from django.conf.urls import url

from .api_views import APIKeyListView, APIKeyView
from .views import (
    KeyDeleteView, KeyDetailView, KeyDownloadView, KeyQueryView,
    KeyQueryResultView, KeyReceive, KeyUploadView, PrivateKeyListView,
    PublicKeyListView
)

urlpatterns = [
    url(
        regex=r'^keys/(?P<key_id>\d+)/$', name='key_detail',
        view=KeyDetailView.as_view()
    ),
    url(
        regex=r'^keys/(?P<key_id>\d+)/delete/$', name='key_delete',
        view=KeyDeleteView.as_view()
    ),
    url(
        regex=r'^keys/(?P<key_id>\d+)/download/$', name='key_download',
        view=KeyDownloadView.as_view()
    ),
    url(
        regex=r'^keys/private/$', name='key_private_list',
        view=PrivateKeyListView.as_view()
    ),
    url(
        regex=r'^keys/public/$', name='key_public_list',
        view=PublicKeyListView.as_view()
    ),
    url(
        regex=r'^keys/upload/$', name='key_upload',
        view=KeyUploadView.as_view()
    ),
    url(
        regex=r'^keys/query/$', name='key_query', view=KeyQueryView.as_view()
    ),
    url(
        regex=r'^keys/query/results/$', name='key_query_results',
        view=KeyQueryResultView.as_view()
    ),
    # Key ID for this view is not the key's DB ID by the embedded
    # alphanumeric ID.
    url(
        regex=r'^keys/receive/(?P<key_id>.+)/$', name='key_receive',
        view=KeyReceive.as_view()
    )
]

api_urls = [
    url(
        regex=r'^keys/(?P<pk>[0-9]+)/$', name='key-detail',
        view=APIKeyView.as_view()
    ),
    url(regex=r'^keys/$', name='key-list', view=APIKeyListView.as_view())
]



from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import Namespace

namespace = Namespace(label=_('Search'), name='search')

setting_search_backend = namespace.add_setting(
    global_name='SEARCH_BACKEND',
    default='mayan.apps.dynamic_search.backends.django.DjangoSearchBackend',
    help_text=_(
        'Full path to the backend to be used to handle the search.'
    )
)
setting_search_backend_arguments = namespace.add_setting(
    global_name='SEARCH_BACKEND_ARGUMENTS',
    default={}
)
setting_disable_simple_search = namespace.add_setting(
    global_name='SEARCH_DISABLE_SIMPLE_SEARCH',
    default=False, help_text=_(
        'Disables the single term bar search leaving only the advanced '
        'search button.'
    )
)



from django.shortcuts import get_object_or_404

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.models import Document
from mayan.apps.rest_api import generics

from .models import ResolvedWebLink, WebLink
from .permissions import (
    permission_web_link_create, permission_web_link_delete,
    permission_web_link_edit, permission_web_link_view,
    permission_web_link_instance_view
)
from .serializers import (
    ResolvedWebLinkSerializer, WebLinkSerializer, WritableWebLinkSerializer
)


class APIDocumentViewMixin:
    def get_document(self):
        queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_web_link_instance_view,
            queryset=Document.objects.all(), user=self.request.user
        )
        return get_object_or_404(klass=queryset, pk=self.kwargs['pk'])


class APIResolvedWebLinkListView(APIDocumentViewMixin, generics.ListAPIView):
    """
    get: Returns a list of resolved web links for the specified document.
    """
    mayan_object_permissions = {'GET': (permission_web_link_instance_view,)}
    serializer_class = ResolvedWebLinkSerializer

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        context = super().get_serializer_context()
        if self.kwargs:
            context.update(
                {
                    'document': self.get_document(),
                }
            )

        return context

    def get_queryset(self):
        return ResolvedWebLink.objects.get_for(
            document=self.get_document(), user=self.request.user
        )


class APIResolvedWebLinkView(APIDocumentViewMixin, generics.RetrieveAPIView):
    """
    get: Return the details of the selected resolved smart link.
    """
    lookup_url_kwarg = 'resolved_web_link_pk'
    mayan_object_permissions = {'GET': (permission_web_link_instance_view,)}
    serializer_class = ResolvedWebLinkSerializer

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        context = super().get_serializer_context()
        if self.kwargs:
            context.update(
                {
                    'document': self.get_document(),
                }
            )

        return context

    def get_queryset(self):
        return ResolvedWebLink.objects.get_for(
            document=self.get_document(), user=self.request.user
        )


class APIResolvedWebLinkNavigateView(
    APIDocumentViewMixin, generics.RetrieveAPIView
):
    """
    get: Perform a redirection to the target URL of the selected resolved smart link.
    """
    lookup_url_kwarg = 'resolved_web_link_pk'
    mayan_object_permissions = {'GET': (permission_web_link_instance_view,)}

    def get_queryset(self):
        return ResolvedWebLink.objects.get_for(
            document=self.get_document(), user=self.request.user
        )

    def retrieve(self, request, *args, **kwargs):
        return self.get_object().get_redirect(
            document=self.get_document(), user=self.request.user
        )

    def get_serializer(self):
        return None

    def get_serializer_class(self):
        return None


class APIWebLinkListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the web links.
    post: Create a new web link.
    """
    mayan_object_permissions = {'GET': (permission_web_link_view,)}
    mayan_view_permissions = {'POST': (permission_web_link_create,)}
    queryset = WebLink.objects.all()

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return WebLinkSerializer
        elif self.request.method == 'POST':
            return WritableWebLinkSerializer


class APIWebLinkView(generics.RetrieveUpdateDestroyAPIView):
    """
    delete: Delete the selected web link.
    get: Return the details of the selected web link.
    patch: Edit the selected web link.
    put: Edit the selected web link.
    """
    lookup_url_kwarg = 'pk'
    mayan_object_permissions = {
        'DELETE': (permission_web_link_delete,),
        'GET': (permission_web_link_view,),
        'PATCH': (permission_web_link_edit,),
        'PUT': (permission_web_link_edit,)
    }
    queryset = WebLink.objects.all()

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return WebLinkSerializer
        else:
            return WritableWebLinkSerializer



from django.http import HttpResponse
from django.shortcuts import get_object_or_404

from rest_framework import status
from rest_framework.generics import get_object_or_404 as rest_get_object_or_404
from rest_framework.response import Response

from mayan.apps.acls.models import AccessControlList
from mayan.apps.common.models import SharedUploadedFile
from mayan.apps.documents.models.document_models import DocumentType
from mayan.apps.documents.permissions import permission_document_create
from mayan.apps.permissions.classes import Permission
from mayan.apps.rest_api import generics
from mayan.apps.storage.classes import DefinedStorage

from .literals import STAGING_FILE_IMAGE_TASK_TIMEOUT, STORAGE_NAME_SOURCE_STAGING_FOLDER_FILE
from .models import StagingFolderSource
from .permissions import (
    permission_sources_setup_create, permission_sources_setup_delete,
    permission_sources_setup_edit, permission_sources_setup_view,
    permission_staging_file_delete
)
from .serializers import (
    StagingFolderFileSerializer, StagingFolderFileUploadSerializer,
    StagingFolderSerializer
)
from .tasks import (
    task_generate_staging_file_image, task_source_handle_upload
)


class APIStagingSourceFileView(generics.RetrieveDestroyAPIView):
    """
    get: Details of the selected staging file.
    """
    serializer_class = StagingFolderFileSerializer

    def get_object(self):
        if self.request.method == 'DELETE':
            Permission.check_user_permissions(
                permissions=(permission_staging_file_delete,),
                user=self.request.user
            )

        staging_folder = get_object_or_404(
            klass=StagingFolderSource, pk=self.kwargs['staging_folder_pk']
        )
        return staging_folder.get_file(
            encoded_filename=self.kwargs['encoded_filename']
        )


class APIStagingSourceListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the staging folders and the files they contain.
    post: Create a new staging folders.
    """
    mayan_view_permissions = {
        'GET': (permission_sources_setup_view,),
        'POST': (permission_sources_setup_create,)
    }
    queryset = StagingFolderSource.objects.all()
    serializer_class = StagingFolderSerializer


class APIStagingSourceView(generics.RetrieveUpdateDestroyAPIView):
    """
    delete: Delete the selected staging folders.
    get: Details of the selected staging folders and the files it contains.
    patch: Edit the selected staging folders.
    put: Edit the selected staging folders.
    """
    mayan_object_permissions = {
        'DELETE': (permission_sources_setup_delete,),
        'GET': (permission_sources_setup_view,),
        'PATCH': (permission_sources_setup_edit,),
        'PUT': (permission_sources_setup_edit,)
    }
    queryset = StagingFolderSource.objects.all()
    serializer_class = StagingFolderSerializer


class APIStagingSourceFileImageView(generics.RetrieveAPIView):
    """
    get: Returns an image representation of the selected staging folder file.
    """
    def get_serializer(self, *args, **kwargs):
        return None

    def get_serializer_class(self):
        return None

    def retrieve(self, request, *args, **kwargs):
        width = request.GET.get('width')
        height = request.GET.get('height')

        task = task_generate_staging_file_image.apply_async(
            kwargs=dict(
                staging_folder_pk=self.kwargs['staging_folder_pk'],
                encoded_filename=self.kwargs['encoded_filename'],
                width=width, height=height
            )
        )

        cache_filename = task.get(timeout=STAGING_FILE_IMAGE_TASK_TIMEOUT)

        storage_staging_file_image_cache = DefinedStorage.get(
            name=STORAGE_NAME_SOURCE_STAGING_FOLDER_FILE
        ).get_storage_instance()

        with storage_staging_file_image_cache.open(cache_filename) as file_object:
            response = HttpResponse(file_object.read(), content_type='image')
            return response


class APIStagingSourceFileUploadView(generics.GenericAPIView):
    """
    post: Upload the selected staging folder file.
    """
    serializer_class = StagingFolderFileUploadSerializer

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        queryset = AccessControlList.objects.restrict_queryset(
            queryset=DocumentType.objects.all(),
            permission=permission_document_create,
            user=self.request.user
        )

        document_type = rest_get_object_or_404(
            queryset=queryset, pk=serializer['document_type'].value
        )

        staging_folder = rest_get_object_or_404(
            queryset=StagingFolderSource, pk=self.kwargs['staging_folder_pk']
        )
        staging_file_object = staging_folder.get_upload_file_object(
            form_data={'staging_file_id': self.kwargs['encoded_filename']}
        )

        shared_uploaded_file = SharedUploadedFile.objects.create(
            file=staging_file_object.file
        )

        task_source_handle_upload.apply_async(
            kwargs={
                'document_type_id': document_type.pk,
                'expand': serializer['expand'].value,
                'shared_uploaded_file_id': shared_uploaded_file.pk,
                'source_id': staging_folder.pk,
                'user_id': self.request.user.pk,
            }
        )

        return Response(status=status.HTTP_202_ACCEPTED)



import logging

from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.acls.links import link_acl_list
from mayan.apps.acls.permissions import permission_acl_edit, permission_acl_view
from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.html_widgets import TwoStateWidget
from mayan.apps.common.menus import (
    menu_list_facet, menu_object, menu_secondary, menu_setup
)
from mayan.apps.navigation.classes import SourceColumn

from .links import (
    link_message_create, link_message_delete, link_message_edit,
    link_message_list
)
from .permissions import (
    permission_message_delete, permission_message_edit,
    permission_message_view
)

logger = logging.getLogger(name=__name__)


class MOTDApp(MayanAppConfig):
    app_namespace = 'motd'
    app_url = 'messages'
    has_rest_api = True
    has_tests = True
    name = 'mayan.apps.motd'
    verbose_name = _('Message of the day')

    def ready(self):
        super(MOTDApp, self).ready()

        Message = self.get_model(model_name='Message')
        ModelPermission.register(
            model=Message, permissions=(
                permission_acl_edit, permission_acl_view,
                permission_message_delete, permission_message_edit,
                permission_message_view
            )
        )
        SourceColumn(
            attribute='label', is_identifier=True, is_sortable=True,
            source=Message
        )
        SourceColumn(
            attribute='enabled', include_label=True, is_sortable=True,
            source=Message, widget=TwoStateWidget
        )
        SourceColumn(
            attribute='start_datetime', empty_value=_('None'),
            include_label=True, is_sortable=True, source=Message
        )
        SourceColumn(
            attribute='end_datetime', empty_value=_('None'),
            include_label=True, is_sortable=True, source=Message
        )

        menu_list_facet.bind_links(
            links=(
                link_acl_list,
            ), sources=(Message,)
        )
        menu_object.bind_links(
            links=(
                link_message_delete, link_message_edit
            ), sources=(Message,)
        )
        menu_secondary.bind_links(
            links=(link_message_create,),
            sources=(Message, 'motd:message_list', 'motd:message_create')
        )
        menu_setup.bind_links(
            links=(link_message_list,)
        )



TEST_CACHE_MAXIMUM_SIZE = 2 * 2 ** 20  # 2 Megabyte
TEST_CACHE_PARTITION_FILE_FILENAME = 'test_cache_partition_file_filename'
TEST_CACHE_PARTITION_FILE_SIZE = 1 * 2 ** 20  # 1 Megabyte
TEST_CACHE_PARTITION_NAME = 'test_cache_partition_name'



from django.apps import apps
from django.utils.encoding import force_text


def get_instance_ocr_content(instance):
    DocumentPageOCRContent = apps.get_model(
        app_label='ocr', model_name='DocumentPageOCRContent'
    )

    for page in instance.pages.all():
        try:
            page_content = page.ocr_content.content
        except DocumentPageOCRContent.DoesNotExist:
            pass
        else:
            yield force_text(page_content)



import json
import logging

from django.db import models, transaction
from django.utils.encoding import force_text
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _

from .classes import NullBackend
from .events import event_quota_created, event_quota_edited

logger = logging.getLogger(__name__)


class Quota(models.Model):
    backend_path = models.CharField(
        max_length=255,
        help_text=_('The dotted Python path to the backend class.'),
        verbose_name=_('Backend path')
    )
    backend_data = models.TextField(
        blank=True, verbose_name=_('Backend data')
    )
    enabled = models.BooleanField(
        default=True, help_text=_(
            'Allow quick disable or enable of the quota.'
        ), verbose_name=_('Enabled')
    )

    class Meta:
        verbose_name = _('Quota')
        verbose_name_plural = _('Quotas')

    def __str__(self):
        return force_text(self.backend_label())

    def save(self, *args, **kwargs):
        _user = kwargs.pop('_user', None)

        with transaction.atomic():
            is_new = not self.pk
            super(Quota, self).save(*args, **kwargs)
            if is_new:
                event_quota_created.commit(
                    actor=_user, target=self
                )
            else:
                event_quota_edited.commit(
                    actor=_user, target=self
                )

    def backend_label(self):
        return self.get_backend_instance().label
    backend_label.help_text = _('Driver used for this quota entry.')
    backend_label.short_description = _('Backend')

    def backend_filters(self):
        return self.get_backend_instance().filters()
    backend_filters.short_description = _('Arguments')

    def backend_usage(self):
        return self.get_backend_instance().usage()
    backend_usage.short_description = _('Usage')

    def dumps(self, data):
        self.backend_data = json.dumps(obj=data)
        self.save()

    def get_backend_class(self):
        """
        Retrieves the backend by importing the module and the class.
        """
        try:
            return import_string(dotted_path=self.backend_path)
        except ImportError as exception:
            logger.error(exception)

            return NullBackend

    def get_backend_instance(self):
        try:
            return self.get_backend_class()(**self.loads())
        except Exception as exception:
            logger.error(exception)

            return NullBackend()

    def loads(self):
        return json.loads(s=self.backend_data)



TEST_DRIVER_INTERNAL_NAME = 'exiftool'
TEST_FILE_METADATA_KEY = 'FileType'
TEST_FILE_METADATA_VALUE = 'PNG'
TEST_FILE_METADATA_INDEX_NODE_TEMPLATE = "{{{{ document.file_metadata_value_of.{}_{} }}}}".format(
    TEST_DRIVER_INTERNAL_NAME, TEST_FILE_METADATA_KEY
)
TEST_PDF_FILE_METADATA_DOTTED_NAME = 'exiftool.Producer'
TEST_PDF_FILE_METADATA_VALUE = 'pdfTeX-1.40.10'



from datetime import timedelta

from django.utils.timezone import now

from mayan.apps.common.settings import settings_db_sync_task_delay

from .events import event_parsing_document_version_submit
from .tasks import task_parse_document_version


def method_document_parsing_submit(self):
    latest_version = self.latest_version
    # Don't error out if document has no version
    if latest_version:
        latest_version.submit_for_parsing()


def method_document_version_parsing_submit(self):
    event_parsing_document_version_submit.commit(
        action_object=self.document, target=self
    )

    task_parse_document_version.apply_async(
        eta=now() + timedelta(seconds=settings_db_sync_task_delay.value),
        kwargs={'document_version_pk': self.pk},
    )



from django.urls import resolve
from django.urls.exceptions import Resolver404

from mayan.apps.common.settings import setting_url_base_path


class Endpoint(object):
    def __init__(self, label):
        self.label = label

        installation_base_url = setting_url_base_path.value
        if installation_base_url:
            installation_base_url = '/{}'.format(installation_base_url)
        else:
            installation_base_url = ''

        url = '{}/api/{}/'.format(installation_base_url, self.label)

        try:
            self.viewname = resolve(path=url).view_name
        except Resolver404:
            self.viewname = None



import logging

from django.utils.module_loading import import_string

from .settings import setting_graphics_backend

logger = logging.getLogger(name=__name__)


def get_converter_class():
    return import_string(dotted_path=setting_graphics_backend.value)



from django.db import connection
from django.utils.translation import ugettext_lazy as _
from mayan.apps.dynamic_search.classes import SearchModel

from .permissions import permission_cabinet_view


def transformation_format_uuid(term_string):
    if connection.vendor in ('mysql', 'sqlite'):
        return term_string.replace('-', '')
    else:
        return term_string


cabinet_search = SearchModel(
    app_label='cabinets', model_name='CabinetSearchResult',
    permission=permission_cabinet_view,
    serializer_path='mayan.apps.cabinets.serializers.CabinetSerializer'
)

cabinet_search.add_model_field(
    field='label', label=_('Label')
)

cabinet_search.add_model_field(
    field='documents__document_type__label', label=_('Document type')
)
cabinet_search.add_model_field(
    field='documents__versions__mimetype', label=_('Document MIME type')
)
cabinet_search.add_model_field(
    field='documents__label', label=_('Document label')
)
cabinet_search.add_model_field(
    field='documents__description', label=_('Document description')
)
cabinet_search.add_model_field(
    field='documents__uuid', label=_('Document UUID'),
    transformation_function=transformation_format_uuid
)
cabinet_search.add_model_field(
    field='documents__versions__checksum', label=_('Document checksum')
)



from mayan.apps.common.tests.base import BaseTestCase

from ..statistics import namespace


class DocumentStatisticsTestCase(BaseTestCase):
    def test_namespace(self):
        for statistic in namespace.statistics:
            try:
                statistic.execute()
            except Exception as exception:
                self.fail(
                    'Error executing: {};  {}'.format(statistic, exception)
                )
                raise



from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link
from mayan.apps.navigation.utils import get_cascade_condition

from .permissions import (
    permission_document_indexing_create, permission_document_indexing_edit,
    permission_document_indexing_delete,
    permission_document_indexing_instance_view,
    permission_document_indexing_rebuild, permission_document_indexing_view
)


def is_not_root_node(context):
    return not context['resolved_object'].is_root_node()


link_document_index_instance_list = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.document_indexing.icons.icon_document_index_instance_list',
    text=_('Indexes'), view='indexing:document_index_list',
)

link_document_type_index_templates = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.document_indexing.icons.icon_document_type_index_templates',
    permissions=(permission_document_indexing_create,), text=_('Index templates'),
    view='indexing:document_type_index_templates',
)

link_index_instance_menu = Link(
    condition=get_cascade_condition(
        app_label='document_indexing', model_name='Index',
        object_permission=permission_document_indexing_instance_view,
    ), icon_class_path='mayan.apps.document_indexing.icons.icon_index',
    text=_('Indexes'), view='indexing:index_list'
)
link_index_instance_rebuild = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.document_indexing.icons.icon_index_instances_rebuild',
    permissions=(permission_document_indexing_rebuild,),
    text=_('Rebuild index'), view='indexing:index_setup_rebuild'
)
link_index_instances_rebuild = Link(
    condition=get_cascade_condition(
        app_label='document_indexing', model_name='Index',
        object_permission=permission_document_indexing_rebuild,
    ),
    icon_class_path='mayan.apps.document_indexing.icons.icon_index_instances_rebuild',
    description=_(
        'Deletes and creates from scratch all the document indexes.'
    ),
    text=_('Rebuild indexes'), view='indexing:rebuild_index_instances'
)
link_index_instances_reset = Link(
    condition=get_cascade_condition(
        app_label='document_indexing', model_name='Index',
        object_permission=permission_document_indexing_rebuild,
    ),
    icon_class_path='mayan.apps.document_indexing.icons.icon_index_instances_reset',
    description=_(
        'Deletes and creates from scratch all the document indexes.'
    ),
    text=_('Reset indexes'), view='indexing:index_instances_reset'
)

link_index_template_setup = Link(
    condition=get_cascade_condition(
        app_label='document_indexing', model_name='Index',
        object_permission=permission_document_indexing_view,
        view_permission=permission_document_indexing_create,
    ), icon_class_path='mayan.apps.document_indexing.icons.icon_index',
    text=_('Indexes'), view='indexing:index_setup_list'
)
link_index_template_list = Link(
    icon_class_path='mayan.apps.document_indexing.icons.icon_index_template_list',
    text=_('Indexes'), view='indexing:index_setup_list'
)
link_index_template_create = Link(
    icon_class_path='mayan.apps.document_indexing.icons.icon_index_template_create',
    permissions=(permission_document_indexing_create,), text=_('Create index'),
    view='indexing:index_setup_create'
)
link_index_template_delete = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.document_indexing.icons.icon_index_template_delete',
    permissions=(permission_document_indexing_delete,), tags='dangerous',
    text=_('Delete'), view='indexing:index_setup_delete',
)
link_index_template_document_types = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.document_indexing.icons.icon_index_template_document_types',
    permissions=(permission_document_indexing_edit,), text=_('Document types'),
    view='indexing:index_setup_document_types',
)
link_index_template_edit = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.document_indexing.icons.icon_index_template_edit',
    permissions=(permission_document_indexing_edit,), text=_('Edit'),
    view='indexing:index_setup_edit',
)

link_index_template_node_tree_view = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.document_indexing.icons.icon_index_template_node_tree_view',
    permissions=(permission_document_indexing_edit,), text=_('Tree template'),
    view='indexing:index_setup_view',
)
link_index_template_node_create = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.document_indexing.icons.icon_index_template_node_create',
    text=_('New child node'),
    view='indexing:template_node_create',
)
link_index_template_node_delete = Link(
    args='resolved_object.pk', condition=is_not_root_node,
    icon_class_path='mayan.apps.document_indexing.icons.icon_index_template_node_delete',
    tags='dangerous', text=_('Delete'), view='indexing:template_node_delete',
)
link_index_template_node_edit = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.document_indexing.icons.icon_index_template_node_edit',
    condition=is_not_root_node, text=_('Edit'),
    view='indexing:template_node_edit',
)



from django.utils.encoding import force_text

from ..permissions import (
    permission_document_edit, permission_document_view
)

from .base import GenericDocumentViewTestCase
from .mixins import (
    DocumentPageDisableViewTestMixin, DocumentPageViewTestMixin
)


class DocumentPageDisableViewTestCase(
    DocumentPageDisableViewTestMixin, GenericDocumentViewTestCase
):
    def test_document_page_disable_view_no_permission(self):
        test_document_page_count = self.test_document.pages.count()

        response = self._request_test_document_page_disable_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            test_document_page_count, self.test_document.pages.count()
        )

    def test_document_page_disable_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_edit
        )

        test_document_page_count = self.test_document.pages.count()

        response = self._request_test_document_page_disable_view()
        self.assertEqual(response.status_code, 302)

        self.assertNotEqual(
            test_document_page_count, self.test_document.pages.count()
        )

    def test_document_page_multiple_disable_view_no_permission(self):
        test_document_page_count = self.test_document.pages.count()

        response = self._request_test_document_page_multiple_disable_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            test_document_page_count, self.test_document.pages.count()
        )

    def test_document_page_multiple_disable_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_edit
        )

        test_document_page_count = self.test_document.pages.count()

        response = self._request_test_document_page_multiple_disable_view()
        self.assertEqual(response.status_code, 302)

        self.assertNotEqual(
            test_document_page_count, self.test_document.pages.count()
        )

    def test_document_page_enable_view_no_permission(self):
        self._disable_test_document_page()

        test_document_page_count = self.test_document.pages.count()

        response = self._request_test_document_page_enable_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            test_document_page_count, self.test_document.pages.count()
        )

    def test_document_page_enable_view_with_access(self):
        self._disable_test_document_page()
        self.grant_access(
            obj=self.test_document, permission=permission_document_edit
        )

        test_document_page_count = self.test_document.pages.count()

        response = self._request_test_document_page_enable_view()
        self.assertEqual(response.status_code, 302)

        self.assertNotEqual(
            test_document_page_count, self.test_document.pages.count()
        )

    def test_document_page_multiple_enable_view_no_permission(self):
        self._disable_test_document_page()
        test_document_page_count = self.test_document.pages.count()

        response = self._request_test_document_page_multiple_enable_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            test_document_page_count, self.test_document.pages.count()
        )

    def test_document_page_multiple_enable_view_with_access(self):
        self._disable_test_document_page()
        self.grant_access(
            obj=self.test_document, permission=permission_document_edit
        )

        test_document_page_count = self.test_document.pages.count()

        response = self._request_test_document_page_multiple_enable_view()
        self.assertEqual(response.status_code, 302)

        self.assertNotEqual(
            test_document_page_count, self.test_document.pages.count()
        )


class DocumentPageViewTestCase(
    DocumentPageViewTestMixin, GenericDocumentViewTestCase
):
    def test_document_page_list_view_no_permission(self):
        response = self._request_test_document_page_list_view()
        self.assertEqual(response.status_code, 404)

    def test_document_page_list_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_document_page_list_view()
        self.assertContains(
            response=response, status_code=200, text=self.test_document.label
        )

    def test_document_page_rotate_left_view_no_permission(self):
        response = self._request_test_document_page_rotate_left_view()
        self.assertEqual(response.status_code, 404)

    def test_document_page_rotate_left_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_document_page_rotate_left_view()
        self.assertEqual(response.status_code, 302)

    def test_document_page_rotate_right_view_no_permission(self):
        response = self._request_test_document_page_rotate_right_view()
        self.assertEqual(response.status_code, 404)

    def test_document_page_rotate_right_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_document_page_rotate_right_view()
        self.assertEqual(response.status_code, 302)

    def test_document_page_view_no_permissions(self):
        response = self._request_test_document_page_view(
            document_page=self.test_document.pages.first()
        )
        self.assertEqual(response.status_code, 404)

    def test_document_page_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_document_page_view(
            document_page=self.test_document.pages.first()
        )
        self.assertContains(
            response=response, status_code=200, text=force_text(
                self.test_document.pages.first()
            )
        )

    def test_document_page_zoom_in_view_no_permission(self):
        response = self._request_test_document_page_zoom_in_view()
        self.assertEqual(response.status_code, 404)

    def test_document_page_zoom_in_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_document_page_zoom_in_view()
        self.assertEqual(response.status_code, 302)

    def test_document_page_zoom_out_view_no_permission(self):
        response = self._request_test_document_page_zoom_out_view()
        self.assertEqual(response.status_code, 404)

    def test_document_page_zoom_out_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_document_page_zoom_out_view()
        self.assertEqual(response.status_code, 302)



from django.utils.translation import ugettext_lazy as _

DEFAULT_IMAP_MAILBOX = 'INBOX'
DEFAULT_IMAP_SEARCH_CRITERIA = 'NOT DELETED'
DEFAULT_IMAP_STORE_COMMANDS = '+FLAGS (\\Deleted)'
DEFAULT_INTERVAL = 600
DEFAULT_METADATA_ATTACHMENT_NAME = 'metadata.yaml'
DEFAULT_POP3_TIMEOUT = 60
DEFAULT_SOURCE_LOCK_EXPIRE = 600
DEFAULT_SOURCE_TASK_RETRY_DELAY = 10

SCANNER_SOURCE_FLATBED = 'flatbed'
SCANNER_SOURCE_ADF = 'Automatic Document Feeder'

SCANNER_SOURCE_CHOICES = (
    (SCANNER_SOURCE_FLATBED, _('Flatbed')),
    (SCANNER_SOURCE_ADF, _('Document feeder')),
)

SCANNER_ADF_MODE_SIMPLEX = 'simplex'
SCANNER_ADF_MODE_DUPLEX = 'duplex'

SCANNER_ADF_MODE_CHOICES = (
    (SCANNER_ADF_MODE_SIMPLEX, _('Simplex')),
    (SCANNER_ADF_MODE_DUPLEX, _('Duplex')),
)

SCANNER_MODE_LINEART = 'lineart'
SCANNER_MODE_MONOCHROME = 'monochrome'
SCANNER_MODE_COLOR = 'color'

SCANNER_MODE_CHOICES = (
    (SCANNER_MODE_LINEART, _('Lineart')),
    (SCANNER_MODE_MONOCHROME, _('Monochrome')),
    (SCANNER_MODE_COLOR, _('Color'))
)

SOURCE_UNCOMPRESS_CHOICE_Y = 'y'
SOURCE_UNCOMPRESS_CHOICE_N = 'n'
SOURCE_UNCOMPRESS_CHOICE_ASK = 'a'

SOURCE_UNCOMPRESS_CHOICES = (
    (SOURCE_UNCOMPRESS_CHOICE_Y, _('Always')),
    (SOURCE_UNCOMPRESS_CHOICE_N, _('Never')),
)

SOURCE_INTERACTIVE_UNCOMPRESS_CHOICES = (
    (SOURCE_UNCOMPRESS_CHOICE_Y, _('Always')),
    (SOURCE_UNCOMPRESS_CHOICE_N, _('Never')),
    (SOURCE_UNCOMPRESS_CHOICE_ASK, _('Ask user'))
)

SOURCE_CHOICE_WEB_FORM = 'webform'
SOURCE_CHOICE_STAGING = 'staging'
SOURCE_CHOICE_WATCH = 'watch'
SOURCE_CHOICE_EMAIL_POP3 = 'pop3'
SOURCE_CHOICE_EMAIL_IMAP = 'imap'
SOURCE_CHOICE_SANE_SCANNER = 'sane'

SOURCE_CHOICES = (
    (SOURCE_CHOICE_SANE_SCANNER, _('Scanner')),
    (SOURCE_CHOICE_WEB_FORM, _('Web form')),
    (SOURCE_CHOICE_STAGING, _('Staging folder')),
    (SOURCE_CHOICE_WATCH, _('Watch folder')),
    (SOURCE_CHOICE_EMAIL_POP3, _('POP3 email')),
    (SOURCE_CHOICE_EMAIL_IMAP, _('IMAP email')),
)
STAGING_FILE_IMAGE_TASK_TIMEOUT = 120
STORAGE_NAME_SOURCE_STAGING_FOLDER_FILE = 'sources__staging_file_image_cache'



from django.utils.translation import ugettext_lazy as _

from mayan.apps.task_manager.classes import CeleryQueue
from mayan.apps.task_manager.workers import worker_medium

queue = CeleryQueue(
    label=_('File metadata'), name='file_metadata', worker=worker_medium
)
queue.add_task_type(
    label=_('Process document version'),
    dotted_path='mayan.apps.file_metadata.tasks.task_process_document_version'
)
